function ksi2i( f, p, q, a2, delta, maxit, iap, ier )

!-----------------------------------------------------------------------
!
!     Computes the inverse of the ksi-square distribution function.
!     That is, given X a random variable distributed according to a
!     ksi-square distribution (with P and Q df's and A2 eccentricity),
!     a probability F, an absolute error DELTA,
!     it returns a value x such that:
!     F - DELTA <= KSI2(x) <= F + DELTA  with KSI2(x) = Proba( X < x )
!
!     F     - Input . Probability                             - Real
!     P     - Input . First degrees of freedom       (P >  0) - Real
!     Q     - Input . Second   "    "     "          (Q >  0) - Real
!     A2    - Input . Eccentricity parameter         (A2>= 0) - Real
!     DELTA - Input . Maximum absolute error required on KSI2 - Real
!                     (stopping criterion)(1e-12 < DELTA < 1)
!     MAXIT - Input . Maximum number of iterations            - Integer
!                     (the maximum number of iterations for
!                     ksi2cdf will be set to 10*maxit)
!     IAP   - Output. 0 if no approximation used              - Integer
!                     1 otherwise
!     IER   - Output. Return code:                            - Integer
!                     0 = no error
!                     1 = invalid input argument
!                         (then KSI2I returns a negative value)
!                     2 = max. iterations reached
!                         (then KSI2I returns the value
!                         at last iteration)
!                     <0= -IOK of KSI2: error in KSI2
!                         (then KSI2I returns a negative value)
!
!     External functions called:
!       KSI2CDF
!
!     Fortran functions called:
!       ABS  LOG  MAX  SQRT
!
!     J. Poitevineau (CNRS-URA1201)  June 03, 1985
!
!-----------------------------------------------------------------------
!
!     Algorithm:
!     Initial solution: mean if Q <= 4
!                     : from a normal distribution if Q > 4.
!     Then, an empirical upper bound is found by incrementing the mean
!     by the standard deviation as long as needed.
!     Then the bounds are iteratively modified by a 2-step process
!     (let Li and Ui the lower and upper bounds at iteration no. i):
!        step1: solution Xi = linear interpolation on interval (Li,Ui)
!        step2: solution Yi = middle of (Xi,Ui) (or (Li,Xi) according
!                             to whether KSI2(Xi) < F or > F),
!               Li+1 = Li (or Yi),  Ui+1 = Yi (or Ui)
!
!     Of course at each stage, whenever KSI2 is called, the result is
!     compared to F and the process is stopped if the solution fits.
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ksi2i

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: f, p, q, a2, delta
   integer, intent(in) :: maxit
   integer, intent(out) :: iap, ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: ksi2cdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: two=one+one, four=two+two
   real(kind=8), parameter :: xsup=1.0e307_8

   !     These constants are machine dependent:
   !     xsup  = stands for +infinity

   real(kind=8) :: c, ex, sx, t, t2, xa, xm, xmx, xz, y, ya, yz
   integer :: iok, it, maxitr

   !--------------------------------------------------------------------

   ksi2i = zero
   iap   = 0
   ier   = 0

   !  Test for valid input arguments

   if ( f < zero .or. f > one .or. a2 < zero .or. p <= zero .or. q <= zero  &
      .or. delta >= one .or. delta <= 1.0e-12_8 ) then
      ier    = 1
      ksi2i  = -xsup
      return
   end if

   !  f near 0 or 1

   if ( f <= delta ) return
   if ( f >= one-delta ) then
      ksi2i = xsup
      return
   end if

   maxitr = 10 * maxit

   !  Compute initial solution xm

   !  ex is mean of ksi2
   y  = one / max(one, q-two)
   ex = a2 + p*q*y
   xm = ex
   if ( q > four ) then
      sx = sqrt(four*a2*q*y+two*p*q*q*y*y*(p+q-two)/(q-four))
      !  From normal distribution, mean ex & std.dev sx
      !  (from IBM-SSP subroutine NDTRI)
      c = f
      if ( c > half ) c = one - c
      t2 = log( one/(c*c) )
      t  = sqrt(t2)
      xm = t - (2.515517_8+0.802853_8*t+0.010328_8*t2)/            &
               (1.0_8+1.432788_8*t+0.189269_8*t2+0.001308_8*t*t2)
      if ( f <= half ) xm = -xm
      xm = xm*sx + ex
   else
      sx = ex
   end if
   if ( xm < zero ) then
      xm = zero
      y  = zero
   else
      y = ksi2cdf( xm, p, q, a2, delta, maxitr, iap, iok )
      if ( iok /= 0 ) goto 30
   end if
   if ( abs(f-y) <= delta ) then
      ksi2i = xm
      return
   end if

   !  Get lower (xa) and upper (xz) bounds for x

   if ( f < y ) then
      !  Get an upper bound
      xz = xm
      yz = y
      xa = zero
      ya = zero
   else
      ! Get a lower bound
      xa = xm
      ya = y
      xz = xsup
      yz = one
      !  Find an upper bound and adjust the lower bound
      xmx = ex
      do
         xmx = xmx + sx
         yz = ksi2cdf( xmx, p, q, a2, delta, maxitr, iap, iok )
         if ( iok /= 0 ) goto 30
         if ( abs(f-yz) <= delta ) then
            ksi2i = xmx
            return
         end if
         if ( yz < f ) then
            xm = xmx
            y  = yz
         else
            exit
         end if
      end do
      xa = xm
      xz = xmx
      ya = y
   end if

   !  Iteration loop

   do it = 1, maxit

      !  1st step: solution xm = linear interpolation on (xa, xz)

      !  Note that there's no protection for xz=xa, f=ya or yz=ya
      !  in the following statement because it should not occur...

      xm = xa + (xz-xa)*(f-ya)/(yz-ya)

      y  = ksi2cdf( xm, p, q, a2, delta, maxitr, iap, iok )
      if ( iok /= 0 ) goto 30
      if ( abs(f-y) <= delta ) then
         ksi2i = xm
         return
      end if
      if ( f < y ) then
         xz = xm
         yz = y
      else
         xa = xm
         ya = y
      end if

      !  2nd step: solution xm = middle of updated (xa, xz)

      xm = (xa+xz) * half
      y  = ksi2cdf( xm, p, q, a2, delta, maxitr, iap, iok )
      if ( iok /= 0 ) goto 30
      if ( abs(f-y) <= delta ) then
         ksi2i = xm
         return
      end if
      if ( f < y ) then
         xz = xm
         yz = y
      else
         xa = xm
         ya = y
      end if

   end do

   !  Maximum number of iterations is reached

   ier   = 2
   ksi2i = xm
   return

   30 continue
   !  Error in  ksi2cdf function

   ier   = -iok
   ksi2i = -xsup

end function ksi2i
