SUBROUTINE eigvct( n, neig, core, eval, evec, wrka, kpvt, ierr )

!***********************************************************************
!     - eigvct -
!     returns the normalized eigenvectors associated to some of the
!     eigenvalues of a given real symmetrical matrix
!***********************************************************************
!
!     N     - INPUT . Order of the symmetrical matrix         - INTEGER
!                     (must be > 1)
!     NEIG  - INPUT . Number of given eigenvalues, thus of    - INTEGER
!                     eigenvectors to return
!     CORE  - INPUT . The symmetrical matrix, of order N,     - REAL*8
!                     stored in triangular form
!     EVAL  - INPUT . The vector of eigenvalues,              - REAL*8
!                     of length NEIG
!     EVEC  - OUTPUT. The matrix of eigenvectors,             - REAL*8
!                     2-dimensioned (N,NEIG)
!                     (the eigenvectors are stored as columns
!                     and they have unit norm)
!     WRKA  - WORK ARRAY of length N*(N-1)/2                  - REAL*8
!     KPVT  - WORK ARRAY of length N-1                        - INTEGER
!     IERR  - OUTPUT. Return code: 0 if normal                - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        DSPFA  DSPSL  (LINPACK subroutines)
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SQRT  SUM
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: n, neig
   REAL(KIND=8), INTENT(IN) :: core(n*(n+1)/2), eval(neig)
   INTEGER, INTENT(OUT) :: ierr
   REAL(KIND=8), INTENT(OUT) :: evec(n,neig)
   REAL(KIND=8), INTENT(OUT) :: wrka(n*(n-1)/2)   ! Work array
   INTEGER, INTENT(OUT) :: kpvt(n-1)              ! Work array

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: one=1.0_8
   REAL(KIND=8) :: vnorm
   INTEGER :: i, ie, ij, j, n1

   !--------------------------------------------------------------------

      ! The eigenvalue is substracted from the diagonal of the core
      ! matrix, then the last coordinate of corresponding eigenvector
      ! is set to -1 so that the last row of the core matrix is taken
      ! as the right hand side vector, thus leading to a classical
      ! symmetrical system of order n-1

   n1 = n - 1

      ! Loop on eigen values/vectors
   do ie = 1, neig

         ! Fill in the symmetrical matrix of the system
      wrka(1) = core(1) - eval(ie)
      ij = 1
      do i = 2, n1
         do j = 1, i-1
            ij = ij + 1
            wrka(ij) = core(ij)
         end do
         ij = ij + 1
         wrka(ij) = core(ij) - eval(ie)
      end do
         ! Fill in the right hand side vector
      evec(1:n1,ie) = core(ij+1:ij+n1)

         ! Solve the system
      call dspfa( wrka, n1, kpvt, ierr )
      if ( ierr /= 0 ) return
      call dspsl( wrka, n1, kpvt, evec(1:n1,ie) )

         ! Normalize the eigenvector
      vnorm = one + sum( evec(1:n1,ie)*evec(1:n1,ie) )
      vnorm = one / sqrt( vnorm )
      evec(1:n1,ie) = evec(1:n1,ie) * vnorm
      evec(n,ie) = -vnorm

   end do

END SUBROUTINE eigvct
