      SUBROUTINE MISQRT( XMAT, DIAG, DUMM, XVEC, N, TOLZ, IERR )
!***********************************************************************
!     - MISQRT -
!     computes the inverse of the positive square-root of a symmetric
!     matrix (positive definite)
!***********************************************************************
!
!     XMAT  - INPUT . The concerned matrix, of order N        - REAL*8
!                     (in triangular form)
!             OUTPUT. Resultant inverted square-root matrix
!                     (in triangular form)
!     DIAG  - WORK VECTOR of length N                         - REAL*8
!     DUMM  - WORK VECTOR of length N                         - REAL*8
!     XVEC  - WORK MATRIX of length N*N                       - REAL*8
!     N     - INPUT . Order of XMAT                           - INTEGER
!     TOLZ  - INPUT . Zero-tolerance for eigenvalues          - REAL*8
!     IERR  - OUTPUT. 0 if no error                           - INTEGER
!                     <0 : xmat was not positive definite:
!                          eigenvalues in DIAG(-IERR)..DIAG(N)
!                          are lower than or equal to zero
!                     >0 : error in diagonalization :
!                          eigenvalues in DIAG(IERR+1)..DIAG(N)
!                          (and corresponding eigenvectors)
!                          may be presumed accurate
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        TQL2X   (adapted from EISPACK library)
!     External functions called:
!        NONE
!     Fortran  functions called:
!        SQRT
!
!-----------------------------------------------------------------------
!
!
!        ARGUMENTS
!        ---------
      DOUBLE PRECISION  TOLZ
      INTEGER           N     , IERR
      DOUBLE PRECISION  XMAT (N*(N+1)/2) , DIAG (N)
      DOUBLE PRECISION  DUMM (N)         , XVEC (N,N)
!
!        LOCAL DECLARATIONS
!        ------------------
      DOUBLE PRECISION  ONE   , SUM   , ZERO
      INTEGER           I     , IJ    , IRANK , J     , K
!
      PARAMETER (ZERO=0.0D0, ONE=1.0D0)
!
!     ------------------------------------------------------------------
!
!
      IERR = 0
!
      IF( N .EQ. 1 )THEN
         IF( XMAT(1) .GT. ZERO )THEN
            XMAT(1) = ONE / SQRT( XMAT(1) )
         else
            IERR = -1
         ENDIF
         RETURN
      ENDIF
!
!     DIAGONALIZE XMAT
!     ----------------
!        (Eigenvectors are returned in XVEC, eigenvalues in DIAG)
      CALL TQL2X( N, XMAT, DIAG, DUMM, XVEC, .TRUE., IERR )
      IF( IERR .NE. 0 ) RETURN
!
!     COMPUTE INVERSE OF SQUARE-ROOT MATRIX ( XVEC*(DIAG**-1/2)*XVEC')
!     -------------------------------------
      IRANK = N
      DO 10 I = 1, N
         IF( DIAG(I) .GT. TOLZ )THEN
            DIAG(I) = ONE / SQRT( DIAG(I) )
         else
!              The input matrix is singular
            IERR  = -I
            RETURN
!              Nonetheless continue processing
!!!!        IRANK = I - 1
!!!!        GOTO 15
         ENDIF
   10 CONTINUE
!!!15 CONTINUE
      IJ = 0
      DO 40 I = 1, N
         DO 30 J = 1, I
            SUM = ZERO
            DO 20 K = 1, IRANK
               SUM = SUM  +  DIAG(K) * XVEC(I,K) * XVEC(J,K)
   20       CONTINUE
            IJ = IJ + 1
            XMAT(IJ) = SUM
   30    CONTINUE
   40 CONTINUE
!
!
      END
