SUBROUTINE msxc( s, x, c, m, n )

!***********************************************************************
!     - msxc -
!     multiplies symmetric matrix S (stored in triangular form) by the
!     rectangular part of subset X of a symmetric matrix (stored in
!     triangular form) to form resultant general matrix C
!***********************************************************************
!
!     S     - INPUT . Symmetric matrix (in triangular form)   - REAL*8
!     X     - INPUT . Subset of a symmetric matrix            - REAL*8
!                     (in partial triangular form)
!     C     - OUTPUT. Resultant general matrix (columnwise)   - REAL*8
!     M     - INPUT . Number of rows and columns in S,        - INTEGER
!                     of rows in C
!     N     - INPUT . Number of columns in C                  - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------
!
!     Explanation :
!     X is a subset of a symmetric matrix (stored in triangular form,
!     of order at least N + M), which has been transmitted from
!     location  N*(N+1)/2 + 1 in calling module (thus X has partial
!     triangular form with length (N+M)*(N+M+1)/2 - N*(N+1)/2).
!     Let  M = 3  and  N = 4, for example. With numbers representing
!     the indices of the elements, X will be:
!         1  2  3  4  5
!         6  7  8  9 10 11
!        12 13 14 15 16 17 18
!     Then this subroutine computes the product of matrix S by the
!     rectangular matrix delimited by indices 1, 4, 15 and 12.
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: m, n
   REAL(KIND=8), INTENT(IN) :: s(m*(m+1)/2), x(m*(m+n+n+1)/2)
   REAL(KIND=8), INTENT(OUT) :: c(m*n)

   !  Local declarations
   !  ------------------

   INTEGER :: i, ij, imm, is, ix, j, k, kj, kx, nm1

   !--------------------------------------------------------------------

      ! Note: the elements of the triangle of the symmetric matrix S
      ! are accessed in their sequential order

   nm1 = n - 1
   ij  = 1 - m

   do j = 1, n
      ij = ij + m
      c(ij) = s(1) * x(j)
   end do

   is = 1
   ix = 0

   do i = 2, m
      is  = is + 1
      imm = i - m
      ij  = -m
      ix  = ix + nm1 + i

      do j = 1, n
         ij = ij + m
         c(ij+i) = s(is) * x(j)
         c(ij+1) = c(ij+1)  +  s(is) * x(ix+j)
      end do

      kx = 0

      do k = 2, i-1
         is = is + 1
         ij = imm
         kj = k - m
         kx = kx + nm1 + k

         do j = 1, n
            ij = ij + m
            kj = kj + m
            c(ij) = c(ij)  +  s(is) * x(kx+j)
            c(kj) = c(kj)  +  s(is) * x(ix+j)
         end do

      end do

      is = is + 1
      ij = imm

      do j = ix+1, ix+n
         ij = ij + m
         c(ij) = c(ij)  +  s(is) * x(j)
      end do

   end do

END SUBROUTINE msxc
