SUBROUTINE formev( oper, nbreva, formul, cmmnt, nomvar, lonvar, x, adr, adrv, result, iwrkb, lerr )

!***********************************************************************
!     - formev -
!     is the directing routine of the formula evaluator:
!     builds the operation tree, and returns the "evaluation" (result)
!     if the formula only contains constants
!***********************************************************************
!
!     OPER  - INPUT . Name of function executing operations   - REAL*8
!     NBREVA- INPUT . Number of different variables           - INTEGER
!     FORMUL- INPUT . Formula (with delimiting apostrophs     - CHAR
!                     removed)
!     CMMNT - INPUT . A logical vector such that CMMNT(I) is  - LOGICAL
!                     .true. if FORMUL(I:I) was inbeded in
!                     apostrophs, and .false. otherwise
!     NOMVAR- INPUT . Names of variables  (dimension NBREVA)  - CHAR
!     LONVAR- INPUT . Length of the names      "        "     - INTEGER
!     X     - OUTPUT. Values of atoms (dimension -1:MAXAT)    - REAL*8
!     ADR   - OUTPUT. (dimension  4, MAXOP)                   - INTEGER
!                     ADR(1,.)  :codes of operators
!                     ADR(2,.)  :index in X of operators
!                     ADR(3/4,.):index in X of operands
!     ADRV  - OUTPUT. (dimension  2, MAXVA)                   - INTEGER
!                     ADRV(1,.) :index in NOMVAR of variables
!                     ADRV(2,.) :index in X of variables
!     RESULT- OUTPUT. Result of evaluation                    - REAL*8
!                     (undefined if NVARF > 0)
!     IWRKB - WORK ARRAY of length 2*NBREVA                   - INTEGER
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     opprm : noper
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        BOUM  FTRIO
!     External functions called:
!        OPER
!     Fortran  functions called:
!        ABS
!
!-----------------------------------------------------------------------
!
!     CALL TREE
!     ----------             BOUM       - atomization of formula
!                               FTRIV   - descending sort
!                               DNUM0   - analysis of constant
!                               VARID1  - identification of variable
!                               VARID2  -        "       "     "
!                            FTRIO      - ascending sort
!                            OPER       - execution of operation
!
! +--------------------------------------------------------------------+
! |           EVALUATEUR DE FORMULE ALGEBRIQUE ET/OU LOGIQUE           |
! |           ----------------------------------------------           |
! |                      Version 2 - 24 Juin 1986                      |
! |                                                                    |
! |  Jacques POITEVINEAU  (CNRS-URA1201: Groupe Math. & Psychologie)   |
! |                                                                    |
! |                                                                    |
! |  PRESENTATION GENERALE                                             |
! |  =====================                                             |
! |                                                                    |
! |    Evaluer une formule c'est lui attribuer une valeur.             |
! |                                                                    |
! |    REMARQUE PRELIMINAIRE                                           |
! |    ---------------------                                           |
! |                                                                    |
! |    Cet evaluateur permet des modifications plus ou moins faciles   |
! |  de certaines options (par exemple le choix des symboles de        |
! |  parenthesage, de caractere de fin de formule,... se fait tout     |
! |  simplement en initialisant certaines variables avant l'appel;     |
! |  par contre, modifier les noms des operateurs, voire leur fonction,|
! |  necessite de remplacer certains modules par un source adequat).   |
! |    De ce fait, une partie de ce qui suit, dependant de ces choix,  |
! |  peut etre inadaptee. par ailleurs, lors des exemples, on se refere|
! |  aux "options standard", a savoir l'utilisation des routines       |
! |  "standard" ce qui implique: le traitement de formules algebriques |
! |  et/ou logiques, avec les operateurs et fonctions classiques sous  |
! |  Fortran, avec eventuellement des alias (LOG et ALOG, par ex.)     |
! |  (sont exclues les fonctions sur les complexes et les fonctions de |
! |  conversion de type). la liste des operateurs "standard" est donnee|
! |  dans le paragraphe utilisation.                                   |
! |                                                                    |
! |    Un peu de vocabulaire... et de syntaxe                          |
! |                                                                    |
! |    Les elements principaux de la formule sont appeles "atomes";    |
! |  un atome peut etre:                                               |
! |     . un operateur  (par exemple: + - * / **    et aussi toute     |
! |                     fonction telle LOG, SIN, ...)                  |
! |                     seuls les operateurs a 1 ou 2 operandes sont   |
! |                     admis dans cette version.                      |
! |                     L'operateur est dit prefixe s'il est precede   |
! |                     d'un operande (meme s'il est aussi suivi       |
! |                     d'un operande, par ex: 3 + 2 );                |
! |                     sinon il est dit postfixe et alors l'usage des |
! |                     parentheses est obligatoire pour entourer l'   |
! |                     (les) operande(s) et s'il y a plusieurs        |
! |                     operandes ils doivent etre separes par un      |
! |                     separateur (la virgule par defaut) par exemple |
! |                     LOG(5) ou MIN(x,3).                            |
! |     . une constante (dont la forme d'ecriture est celle de Fortran:|
! |                     elle peut etre precedee des signes + ou -,     |
! |                     le point represente la virgule decimale,       |
! |                     elle peut etre sous forme exponentielle, E ou D|
! |                     symbolisant "10 puissance", par ex: 0.0123     |
! |                     peut s'ecrire 1.23E-2, 123E-4, ...).           |
! |     . une variable  (reperee par un nom figurant dans une liste    |
! |                     donnee des noms des variables). n.b. si le     |
! |                     nom commence par un chiffre ou par un nom de   |
! |                     fonction, il doit obligatoirement etre entoure |
! |                     d'apostrophes (meme autrement il est preferable|
! |                     de l'entourer d'apostrophes car cela reduit    |
! |                     les risques d'erreur et accelere l'execution). |
! |                                                                    |
! |    Les parentheses ne sont pas considerees comme des atomes, mais  |
! |  elles jouent bien sur un role important puisqu'elles influent sur |
! |  le niveau hierarchique des operateurs.                            |
! |                                                                    |
! |    Pour l'essentiel les regles d'ecriture d'une formule sont les   |
! |  regles classiques sous Fortran :                                  |
! |  (ci-dessous, les regles dependantes de choix sont precedees de +  |
! |  les autres etant precedees de - ).                                |
! |                                                                    |
! |  - Les parentheses sont optionnelles tant qu'il n'y a pas d'erreur |
! |    possible sur l'ordre des operations a effectuer.                |
! |    (n.b. pour effectuer 3 fois -2 il faut ecrire  3*(-2) et non    |
! |    3*-2 qui serait mal interprete. d'une facon generale, deux      |
! |    operateurs prefixes ne peuvent se suivre).                      |
! |  - Les operations sont executees selon le niveau hierarchique:     |
! |    l'operateur de niveau le plus eleve est execute en premier,etc. |
! |    a niveau hierarchique egal, les operations sont effectuees de   |
! |    la gauche vers la droite.                                       |
! |  + Le niveau hierarchique des operateurs est, par ordre croissant  |
! |    (les 5 premiers niveaux concernant les operateurs logiques) :   |
! |       1) equivalence et non equivalence                            |
! |       2) ou inclusif                                               |
! |       3) et                                                        |
! |       4) negation                                                  |
! |       5) relationnels (egal, non egal, superieur, etc)             |
! |       6)    addition    et soustraction  et signe (+ -)            |
! |       7) multiplication et division                                |
! |       8) puissance                                                 |
! |       9) toutes les fonctions representees par un nom              |
! |          (comme LOG, LOG10, EXP, SQRT, SIN, ...)                   |
! |  - Les blancs ne sont pas significatifs et peuvent donc servir a   |
! |    aerer la presentation de la formule.                            |
! |                                                                    |
! |    Mais avec les differences suivantes par rapport a Fortran :     |
! |                  ---------------------                             |
! |                                                                    |
! |  + La formule doit forcemment se terminer par le caractere "SEP2"  |
! |  - Les constantes et le resultat sont definis en "double precision"|
! |    sans qu'il soit necessaire de le preciser par l'utilisation du  |
! |    "." ou d'un exposant (mais cela reste possible) :               |
! |    ainsi 3 est bien admis comme un "reel", ce qui permet d'ecrire  |
! |    SQRT(3) pour racine carree de 3 (alors qu'en Fortran il faudrait|
! |    SQRT(3.) ou SQRT(3E0) ou DSQRT(3D0) ).                          |
! |  - De meme toutes les fonctions sont en "double precision",        |
! |    independamment de la lettre de debut de nom.                    |
! |    en consequence, aux noms de fonction comme ALOG, DLOG, AMIN1,...|
! |    il est possible (et fortement conseille, pour plus de rapidite; |
! |    voire obligatoire, selon les options choisies) de substituer    |
! |    les noms generiques LOG, MIN,...                                |
! |  - Les fonctions min et max ne peuvent avoir plus de 2 operandes.  |
! |    (si besoin est, on peut toujours faire, par ex. pour MIN(X,Y,Z) |
! |    MIN(X,(MIN(Y,Z)) ).                                             |
! |  + Les valeurs logiques sont 1 pour vrai (au lieu de .TRUE.) et    |
! |    0 pour faux (au lieu de .FALSE.).                               |
! |  + Les operateurs logiques sont delimites par le signe _ au lieu   |
! |    du point (ex:  _OR_ ).                                          |
! |  + Comme en Fortran une expression logique peut contenir des       |
! |    expressions algebriques (par ex.  X+Y _GT_ 3 ), mais on peut    |
! |    egalement (a ses risques et perils) introduire des operateurs   |
! |    logiques dans une formule algebrique : les expressions          |
! |    algebriques connectees par un operateur logique non relationnel |
! |    (par ex. _NOT_, _AND_, etc) sont alors converties a 1 si elles  |
! |    sont strictement positives, a 0 sinon. ainsi : 3 * (_NOT_(X+Y)) |
! |    sera accepte et donnera 0 si X+Y > 0, et 3 sinon.               |
! |                                                                    |
! |  ALGORITHME GENERAL                                                |
! |  ==================                                                |
! |                                                                    |
! |  1- Apres son entree la formule est comprimee (suppression des     |
! |    blancs) et decomposee en ses atomes.                            |
! |    Chaque atome est numerote selon son ordre d'apparition dans la  |
! |    formule (numerotation croissante). dans la suite ce numero est  |
! |    appele "adresse" de l'atome.                                    |
! |                                                                    |
! |     Le tableau X est associe aux atomes: le ieme element de X      |
! |    contient la valeur du ieme atome de la formule si celui-ci est  |
! |    une constante ou une variable, 0 si c'est un operateur.         |
! |    (on appellera NOPER le nombre total d'operateurs de la formule, |
! |    qu'ils soient differents ou non).                               |
! |                                                                    |
! |     On calcule le niveau hierarchique definitif des operateurs en  |
! |    fonction de leur hierarchie intrinseque et du niveau de         |
! |    parenthesage.                                                   |
! |    Ces niveaux sont stockes dans le tableau NIVOP: NIVOP(i)        |
! |    contient le niveau du ieme operateur apparu.                    |
! |    Les codes numeriques et les adresses dans X des operateurs sont |
! |    respectivement stockes dans les tableaux ADR(1,.) et ADR(2,.).  |
! |               *** subroutine BOUM ***                              |
! |                                                                    |
! |  2- NIVOP est trie selon l'ordre hierarchique croissant (en tenant |
! |    compte de la preseance eventuelle gauche/droite) (NIVOP(1)      |
! |    contient alors le niveau du dernier operateur a executer,       |
! |    NIVOP(NOPER) celui du premier operateur a executer),            |
! |    ADR(1,.) ADR(2,.) etant reordonnes en parallele (i.e. ADR(2,1)  |
! |    contient l'adresse de l'operateur execute en dernier, etc).     |
! |               *** subroutine FTRIO ***                             |
! |                                                                    |
! |  3- On construit les tableaux ADR(3/4,.) associes a ADR(2,.) :     |
! |      ADR(3,i) contient ADR(2,i)-1 si le ieme operateur (selon      |
! |      l'ordre hierarchique) est prefixe, et ADR(2,i)+1 sinon.       |
! |      ADR(4,i) contient, respectivement, ADR(2,i)+1 ou ADR(2,i)+2   |
! |      (ou -1 si l'operateur n'agit que sur un seul argument, comme  |
! |      la fonction LOG par exemple).                                 |
! |    Autrement dit ADR(3/4,.)contiennent ce que seraient les adresses|
! |    des operandes des operateurs si on ne tenait pas compte de leur |
! |    hierarchie. (on notera que ces adresses sont forcemment         |
! |    correctes en ce qui concerne l'operateur le plus haut dans la   |
! |    hierarchie, i.e. celui dont l'adresse figure dans ADR(2,NOPER)).|
! |               *** subroutine FORMEV ***                            |
! |                                                                    |
! |  4- On construit l'arbre des operations                            |
! |    (en fin de processus ADR(3,i), ADR(4,i) contiendront les        |
! |    adresses des noeuds "enfants" de ADR(2,i).) :                   |
! |      On itere pour i allant de NOPER a 2 :                         |
! |        dans un vecteur reserve (WRK) les adresses valant ADR(3,i)  |
! |        ou ADR(4,i) sont remplacees par ADR(2,i). ce vecteur sert a |
! |        la mise a jour des adresses.                                |
! |        On itere pour j allant de i-1 a 1 :                         |
! |          ADR(3,j) et ADR(4,j) sont mises a jour (=sont remplacees  |
! |          par le contenu correspondant de WRK).                     |
! |          (dans certains cas: ADR(4,j)=ADR(3,j); i.e. le 1er        |
! |          argument d'un operateur a plusieurs arguments est lui-meme|
! |          une expresion. il faut alors retablir l'adresse de l'     |
! |          argument suivant: ADR(4,j)=ADR(4,j) + |nbre operandes de  |
! |          adr(1,i)|; puis ADR(4,j) est mis a jour;  si on retrouve  |
! |          l'egalite precedente on itere le processus :              |
! |          ADR(4,j)=ADR(4,j)+1; mise a jour de ADR(4,j);             |
! |          jusqu'a ce qu'elle ne soit plus verifiee.                 |
! |               *** subroutine FORMEV ***                            |
! |                                                                    |
! |  5- On effectue les operations                                     |
! |      On itere pour i allant de noper a 2 :                         |
! |      X(ADR(2,i))= operateur execute sur X(ADR(3,i)) et X(ADR(4,i)) |
! |    (N.B. si la formule ne contient pas de variable, l'execution    |
! |    se fait en parallele a la construction de l'arbre, sinon elle   |
! |    est precedee par une etape de mise a jour des valeurs des       |
! |    variables et de mise a 0 des X correspondants aux operateurs.   |
! |    Ceci est utile dans le cas ou une meme formule doit etre        |
! |    calculee pour plusieurs "individus", seules changeant les       |
! |    valeurs des variables selon les individus. Ainsi l'arbre est    |
! |    construit une fois pour toute.)                                 |
! |               *** subroutine FORMEV ou FORMAJ ***                  |
! |                                                                    |
! |  6- Le resultat final est obtenu en executant l'operateur de plus  |
! |    faible hierarchie sur X(ADR(3,1)) et X(ADR(4,1)).               |
! |               *** subroutine FORMEV ou FORMAJ ***                  |
! |                                                                    |
! |  ADAPTATIONS POSSIBLES                                             |
! |  =====================                                             |
! |                                                                    |
! |    Dans le cadre "standard" (le programmeur ne change aucun        |
! |  sous-programme), on peut facilement choisir: les operateurs       |
! |  (algebriques et/ou logiques, avec/sans alias), les caracteres     |
! |  speciaux (fin de formule, parenthesage, separateur d'operandes,   |
! |  virgule decimale).                                                |
! |                                                                    |
! |    Et avec un peu plus d'efforts :                                 |
! |  on peut modifier, supprimer ou ajouter des operateurs, choisir    |
! |  leurs symboles ou noms et leurs fonctions (entr'autre, il pourrait|
! |  etre utile de modifier les conversions des valeurs algebriques    |
! |  dans les cas d'expression algebriques avec operateurs logiques).  |
! |  Les operateurs pourraient avoir plus de deux operandes (dans ce   |
! |  cas ils seraient necessairement postfixes).                       |
! |                                                                    |
! |  UTILISATION                                                       |
! |  ===========                                                       |
! |                                                                    |
! |    Les sous-programmes utilises sont :                             |
! |  FORMEV BOUM FTRIV DNUM0 VARID1 VARID2 FTRIO FOPER FORMAJ et le    |
! |  sous-programme block data OPINIT (initialisation de commons).     |
! |                                                                    |
! |       I - CAS STANDARD : PAS DE MODIFICATION DES ROUTINES          |
! |       ----------------                                             |
! |                                                                    |
! |    Les symboles et noms de fonctions sont :                        |
! |                                                                    |
! |  Logiques    :  _EQV_    _NEQV_   _OR_     _AND_    _NOT_          |
! |                 _GT_     _GE_     _LT_     _LE_     _EQ_           |
! |                 _NE_                                               |
! |  Algebriques :  +        -        /        *        **             |
! |                 SQRT     LOG      LOG10    EXP      ABS            |
! |                 INT      NINT     SIGN     MOD      MIN            |
! |                 MAX      SIN      SINH     COS      COSH           |
! |                 TAN      TANH     COTAN    ASIN     ACOS           |
! |                 ATAN     ATAN2    GAMMA    LGAMMA   ERF            |
! |                 ERFC     DIM      DEG      RAD      &X             |
! |                 :=:                                                |
! |  (&X est la fonction speciale representant la valeur d'une variable|
! |  en fonction de son indice: &X(i) = valeur de la ieme variable)    |
! |  (:=: est la fonction speciale "texte", representation en double   |
! |  precision d'une chaine de caracteres, tronquee a ses 8 premiers   |
! |  elements)                                                         |
! |    Avec, eventuellement, les alias :                               |
! |  Pour _NOT_ a _NE_ :  ^   >   >=   <    <=   =   ^=                |
! |  DSQRT  ALOG   DLOG   ALOG10 DLOG10 DEXP   IABS   DABS   AINT      |
! |  DINT   ANINT  DNINT  ISIGN  DSIGN  AMOD   DMOD   AMIN0  AMIN1     |
! |  DMIN1  AMAX0  AMAX1  DMAX1  DSIN   DSINH  DCOS   DCOSH  DTAN      |
! |  DTANH  DCOTAN DASIN  DACOS  DATAN  DATAN2 DGAMMA ALGAMA DLGAMA    |
! |  DERF   DERFC  IDIM   DDIM                                         |
! |                                                                    |
! |  1- Dans le programme appelant, declarer les commons /OPNOM/,      |
! |     /OPLHN/, /OPPRM/ et les tableaux et variables necessaires.     |
! |     initialiser les variables IO, MAXAT, MAXOP, MAXVA, du common   |
! |     /OPPRM/, ainsi que IDBOP, IFNOP, ITXOP, KFIN eventuellement.   |
! |  2- Pour chaque formule a evaluer:                                 |
! |     appel de FORMEV pour construire l'arbre des operations.        |
! |     a) elle ne contient aucune variable (i.e. NVARF=0): le resultat|
! |        est directement donne en retour de FORMEV.                  |
! |     b) elle contient des variables (i.e. NVARF>0):                 |
! |        a chaque fois que des valeurs sont attribuees aux variables |
! |        appel de FORMAJ pour avoir le resultat.                     |
! |                                                                    |
! |  REMARQUE - C'est au programmeur de verifier qu'il y a bien une    |
! |  formule a evaluer: FORMEV ne teste pas que la chaine transmise    |
! |  est nulle ou toute a blanc.                                       |
! |                                                                    |
! |       II- AVEC ADAPTATIONS                                         |
! |       --------------------                                         |
! |                                                                    |
! |    En plus des 2 etapes du cas precedent, il faut:                 |
! |                                                                    |
! |  1- Adapter NBREOP dans les instructions parameter                 |
! |     et le tableau KOPER dans le sous-programme BOUM                |
! |  2- Redefinir le blockdata OPINIT                                  |
! |     (-important- si le nom d'un operateur est aussi le  prefixe    |
! |     d'autres noms, tous ces operateurs doivent etre contigus dans  |
! |     la liste des operateurs - cf parameter "MEME" dans le sous-    |
! |     programme BOUM)                                                |
! |  3- Redefinir la fonction d'execution (appelee FOPER en standard)  |
! |  4- Eventuellement, dans le sous-programme BOUM, modifier la       |
! |     matrice SERR des erreurs de succession.                        |
! |                                                                    |
! +--------------------------------------------------------------------+

   USE oplhn
   USE opprm

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), EXTERNAL :: oper   ! The function executing operations
   INTEGER, INTENT(IN) :: nbreva
   CHARACTER(LEN=*), INTENT(IN) :: formul
   INTEGER, DIMENSION(nbreva), INTENT(IN) :: lonvar
   LOGICAL, DIMENSION(LEN(formul)), INTENT(IN) :: cmmnt
   CHARACTER(LEN=*), DIMENSION(nbreva), INTENT(IN) :: nomvar
   REAL(KIND=8), INTENT(OUT) :: result
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, DIMENSION(4,maxop), INTENT(OUT) :: adr
   INTEGER, DIMENSION(2,maxva), INTENT(OUT) :: adrv
   INTEGER, DIMENSION(nbreva+nbreva), INTENT(OUT) :: iwrkb
   REAL(KIND=8), DIMENSION(-1:maxat), INTENT(OUT) :: x

   !  Local declarations
   !  ------------------

   INTEGER :: i, j, kjad, nter
   LOGICAL :: lnovar
   INTEGER, DIMENSION(-1:maxat) :: wrk

   ! wrk -  work vector used both for:
   !      . hierarchy of operators (dimension  1:maxop)
   !      . addresses updating     (   "      -1:maxat)

   !--------------------------------------------------------------------

   lerr = .false.

   ! Atomization of formula -> x, wrk, adr, adrv
   ! -------------------------------------------
   call boum( nbreva, formul, cmmnt, x, wrk(1:), adr,             &
              adrv, nomvar, lonvar, iwrkb, iwrkb(nbreva+1:), lerr )
   if ( lerr ) return

   ! Case of a formula without any operator: interpreted as an
   ! assignement (code no. nbreop = unary +) of a const./variable
   ! ------------------------------------------------------------
   if ( noper == 0 ) then
      noper = 1
      adr(1,1) = nbreop
      adr(2,1) = 0
   end if

   ! Sort wrk. adr(1/2,.) reordered accordingly
   ! ------------------------------------------
   call ftrio( noper, wrk(1:), adr )

   ! Initialization for adress changes
   ! ---------------------------------
   do i = -1, natom
      wrk(i) = i
   end do

   ! Initial computation of adr
   ! --------------------------
      ! If operator i is prefixed then adr(3,i) and adr(4,i) are the
      ! adresses immediately preceding and following the address of
      ! operator; otherwise they are the 2 following adresses and if
      ! there is only 1 operand adr(4,i) is set to -1
   do i = 1, noper
      adr(4,i) = -1
      if ( nopop(adr(1,i)) < 0 ) then
         adr(3,i) = adr(2,i) - 1
         if ( nopop(adr(1,i)) < -1 ) adr(4,i) = adr(2,i) + 1
      else
         adr(3,i) = adr(2,i) + 1
         if ( nopop(adr(1,i)) .gt. 1 ) adr(4,i) = adr(2,i) + 2
      end if
   end do

   ! adr is updated, and operation is executed if no variable
   ! --------------------------------------------------------
   ! Note that the last 5 arguments of oper are not relevant here
   nter   = 0
   lnovar = nvarf == 0 .and. index( formul, nomop(ivrop)(:lonop(ivrop)) ) <= 0
   do i = noper, 2, -1

      ! Execution of operation if no variable in formula
      if ( lnovar ) x(adr(2,i)) = oper( adr(1,i), x( adr(3,i) ), x( adr(4,i) ), x, 1, 0, 0, nter )

      ! Array of address changes is updated
      wrk(adr(3,i)) = adr(2,i)
      if ( adr(4,i) >= 0 ) wrk(adr(4,i)) = adr(2,i)
      do j = 0, natom
         wrk(j) = wrk(wrk(j))
      end do

      ! adr is updated ( -1 are unchanged)
      do j = i-1, 1, -1
         adr(3,j) = wrk(adr(3,j))
         if ( adr(4,j) >= 0 ) then
            kjad     = adr(4,j)
            adr(4,j) = wrk(kjad)
            ! Case of postfixed operators which 1st argument is
            ! itself an expression (then the address of 2nd argument
            ! must be updated).
            ! This case is characterized by adr(3,j)=adr(4,j)
            if ( adr(4,j) == adr(3,j) ) then
               adr(4,j) = kjad + abs(nopop(adr(1,i)))
               kjad     = adr(4,j)
               adr(4,j) = wrk(kjad)
                  ! If necessary, iterations are performed
               if ( adr(4,j) == adr(3,j) ) then
                  do
                     kjad     = kjad + 1
                     adr(4,j) = wrk(kjad)
                     if ( adr(4,j) /= adr(3,j) ) exit
                  end do
               end if
            end if
         end if
      end do

   end do

   ! Final result if no variable in the formula
   ! ------------------------------------------
   ! Note that the last 5 arguments of oper are not relevant here
   if ( lnovar ) result = oper( adr(1,1), x(adr(3,1)), x(adr(4,1)), x, 1, 0, 0, nter )

END SUBROUTINE formev
