SUBROUTINE getuvw( helm, metr1, type, ng, nt, nvp, nwp, gpar, tpar, part,   &
                   gwgh, twgh, vwgh, wwgh, v, maxv, w, maxw )
!***********************************************************************
!     - getuvw -
!     GETs the contrast coefficients associated to a comparison without
!     any interaction sign: U contrast for a 1 df non-product form
!     comparison or V and W marginal contrasts for a product form one
!***********************************************************************
!
!     HELM  - INPUT . .true. if an Helmert's contrast is      - LOGICAL
!                     desired
!     METR1 - INPUT . Relevant only if type is 1 or 2 and     - LOGICAL
!                     HELM is .false.
!                     If .true. the unit metric will be used
!                     to get orthogonal contrasts; if .false.
!                     the metric used will be reciprocal of
!                     VWGH or WWGH, according to type
!     TYPE  - INPUT . Type of the comparison:                 - INTEGER
!                        1 = W/G1 type        (product form)
!                        2 = V/T1 type        (product form)
!                        3 = V.W  type (1 df) (product form)
!                        4 = general type (non product form)
!     NG    - INPUT . Number of groups                        - INTEGER
!     NT    - INPUT . Number of treatments                    - INTEGER
!     NVP   - INPUT . Number of different parts in GPAR       - INTEGER
!     NWP   - INPUT . Number of different parts in TPAR       - INTEGER
!     GPAR  - INPUT . G-partition vector, of length NG        - INTEGER
!     TPAR  - INPUT . T-partition vector, of length NT        - INTEGER
!     PART  - INPUT . Partition matrix, 2-dimensioned (NT,NG) - INTEGER
!     GWGH  - INPUT . Vector of G-weights of length NG        - REAL*8
!                     (unit vector if equal weighting option,
!                     group counts if size weighting option)
!                     Used only if TYPE=3 or HELM=.true.
!     TWGH  - INPUT . Vector of T-weights of length NT        - REAL*8
!                     (unit vector if equal weighting option)
!                     Used only if TYPE=3 or HELM=.true.
!     VWGH  - INPUT . Vector of G-part-weights of length NVP  - REAL*8
!                     Used only if TYPE=2 and METR1=.false.
!                     and HELM=.false.
!     WWGH  - INPUT . Vector of T-part-weights of length NWP  - REAL*8
!                     Used only if TYPE=1 and METR1=.false.
!                     and HELM=.false.
!     V     - OUTPUT. Matrix of V-contrast coefficients       - REAL*8
!                     2-dimensioned (NG,MAXV)
!                     Irrelevant if type is 4
!     MAXV  - INPUT . Maximum number of columns of array V    - INTEGER
!     W     - OUTPUT. Matrix of W-contrast coefficients       - REAL*8
!                     2-dimensioned (NT,MAXW)
!     MAXW  - INPUT . Maximum number of columns of array W    - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        BASORT  GMP999  PAC999
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX
!
!-----------------------------------------------------------------------

   USE pacint

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   LOGICAL, INTENT(IN) :: helm, metr1
   INTEGER, INTENT(IN) :: type, ng, nt, nvp, nwp, maxv, maxw
   INTEGER, INTENT(IN) :: gpar(ng), tpar(nt), part(nt,ng)
   REAL(KIND=8), INTENT(IN) :: gwgh(ng), twgh(nt), vwgh(nvp), wwgh(nwp)
   REAL(KIND=8), INTENT(OUT) :: v(ng,maxv), w(nt,maxw)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   INTEGER :: i, j, k, l
   REAL(KIND=8) :: sumx(0:2)
   REAL(KIND=8) :: cox(0:5)=(/ zero, one, -one, zero, zero, zero /)
   REAL(KIND=8) :: wrk(0:max(ngrp,ntrt)*(max(ngrp,ntrt,2)-1))
                 ! wrk is work array of dimension 0:m*(m-1) where m is
                 !       nwp if type is 1
                 !       nvp if type is 2
                 !       Irrelevant otherwise or if helm=.false.

   !--------------------------------------------------------------------

   l = 0
      ! Get the relevant coefficient if Helmert's contrast
   if ( helm ) then
      l = 3
      cox(4) = zero
      cox(5) = zero
      do j = 1, ng
         do i = 1, nt
            if ( part(i,j) > 0 ) cox(3+part(i,j)) = cox(3+part(i,j)) + twgh(i)*gwgh(j)
         end do
      end do
      cox(3) = zero
      cox(4) = cox(5) / ( cox(4)+cox(5) )
      cox(5) = -cox(4)
   end if

   select case (type)

      case default ! Invalid type (should not occur)
         call gmp999
         call pac999( 99 )
         stop 'GETUVW'

      case (1)     ! W/G1 type
                   ! ---------

            ! Get W contrast vectors
         if ( helm ) then
            w(1:nt,1) = cox( 3+tpar(1:nt) )
         else
               ! Find an orthonormal basis set for W (stored in wrk(1) )
            call basort( nwp, nwp, metr1, wwgh, wrk(1:nwp*max(nwp-1,1)) )
               ! Fill in W contrast vectors
            k = 0
            do j = 1, max(nwp-1, 1)
               wrk(k) = zero
               w(1:nt,j) = wrk( k+tpar(1:nt) )
               k = k + nwp
            end do
         end if
            ! Get unique V
         v(1:ng,1) = cox( gpar(1:ng) )

      case (2)     ! V/T1 type
                   ! ---------

            ! Get V contrast vectors
         if ( helm ) then
            v(1:ng,1) = cox( 3+gpar(1:ng) )
         else
               ! Find an orthonormal basis set for V (stored in wrk(1))
            call basort( nvp, nvp, metr1, vwgh, wrk(1:nvp*max(nvp-1,1)) )
               ! Fill in V contrast vectors
            i = 0
            do j = 1, max(nvp-1, 1)
               wrk(i) = zero
               v(1:ng,j) = wrk( i+gpar(1:ng) )
               i = i + nvp
            end do
         end if
            ! Get unique W
         w(1:nt,1) = cox( tpar(1:nt) )

      case (3)     ! 1 df V*W special type (thus only 2 parts)
                   ! ---------------------

         w(1:nt,1) = cox( l+tpar(1:nt) )
         sumx(1) = zero
         sumx(2) = zero
         do k = 1, ng
            j = gpar(k)
            if ( j > 0 ) sumx(j) = sumx(j) + gwgh(k)
         end do
         sumx(0) = sumx(1) + sumx(2)
         sumx(1) = sumx(1) / sumx(0)
         sumx(2) = sumx(2) / sumx(0)
         sumx(0) = zero
         v(1:ng,1) = cox( gpar(1:ng) ) * sumx( gpar(1:ng) )

      case (4)     ! 1 df general type (non product form)
                   ! ------------------------------------

      do j = 1, ng
         do i = 1, nt
            w(i,j) = cox( l + part(i,j) )
         end do
      end do

   end select

END SUBROUTINE getuvw
