SUBROUTINE pacadj( ybar, zbar, slop, ag, supg, sumw, nwdep, nwcov )

!***********************************************************************
!     - pacadj -
!     computes means of dependent variables ADJusted for covariates.
!     Note it is assumed coefficients of each w-contrast sum to zero
!     when ncw is greater than one and sum to zero or one otherwise
!***********************************************************************
!
!     YBAR  - INPUT . Matrix of Y-means                       - REAL*8
!             OUTPUT. Adjusted means
!     ZBAR  - OUTPUT. Vector of means of covariates over all  - REAL*8
!                     groups of G-support, multiplied by SUMW
!                     if NCW is 1, a zero-vector otherwise;
!                     of length nwcov
!     SLOP  - INPUT . Matrix of slopes of covariates          - REAL*8
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     SUPG  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     SUMW  - INPUT . Sum of W-contrast coefficients if NCW=1 - REAL*8
!                     (whether zero or one)
!     NWDEP - INPUT . NCW * NDEP, number of columns in SLOP   - INTEGER
!     NWCOV - INPUT . NCW * NCOVA, number of rows in SLOP     - INTEGER
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        NONE
!
!-----------------------------------------------------------------------

   USE pacint

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: sumw
   INTEGER, INTENT(IN) :: nwdep, nwcov
   REAL(KIND=8), INTENT(IN) :: slop(nwcov,nwdep), ag(ngrp)
   INTEGER, INTENT(IN) :: supg(ngsup)
   REAL(KIND=8), INTENT(INOUT) :: ybar(nwdc,ngsup)
   REAL(KIND=8), INTENT(OUT) :: zbar(nwcov)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: sumx, tot
   INTEGER :: i, j, k, ks

   !--------------------------------------------------------------------

   ! Compute zbar
   ! ------------
   if ( ncw > 1 .or. sumw == zero ) then
         ! The sum of w-coefficients is zero for each w-contrast
         ! so zbar is a zero vector
      zbar = zero

   else
         ! ncw is 1 and the sum of w-coefficients (sumw) is one
         ! so zbar = means of covariates over all relevant groups
         ! (according to relevant weighting)
      if ( gpond == 1 ) then
         tot = ngsup
         do i = 1, nwcov
            j = nwdep + i
            zbar(i) = sum(ybar(j,:)) / tot
         end do
      else
         do i = 1, nwcov
            j = nwdep + i
            sumx = zero
            tot = zero
            do ks = 1, ngsup
               k = supg(ks)
               sumx = sumx + ag(k) * ybar(j,ks)
               tot = tot  +  ag(k)
            end do
            zbar(i) = sumx / tot
         end do
      end if

   end if

   ! Computes adjusted means of dependent variables
   ! ----------------------------------------------
   do ks = 1, ngsup

      do j = 1, nwdep
         sumx = zero
         do i = 1, nwcov
            sumx = sumx + (ybar(nwdep+i,ks) - zbar(i)) * slop(i,j)
         end do
         ybar(j,ks) = ybar(j,ks) - sumx
      end do

   end do

END SUBROUTINE pacadj
