SUBROUTINE pacalf( form, lenf, allgt, balang, balant,              &
                   flag, maxv, maxw, lexs, rcoef, nrco,            &
                   type, rtyp, kdeg, nrsavd, nregc, intcp, prop,   &
                   ndf1, zinter, iord, nvp1, nwp1, npart,          &
                   gpar, tpar, part, vv, ww, lerr )
!***********************************************************************
!     - pacalf -
!     Analyzes the comparison: the "Left part" of Formula,
!     i.e. a sequence   a.b.c etc   where a, b, c are some "between
!     parts comparisons".
!     The analysis results in matrices of contrast-coefficients VV, WW
!***********************************************************************
!
!     FORM  - INPUT . Formula                                 - CHAR
!     LENF  - INPUT . Length of the "left part" of formula    - INTEGER
!     ALLGT - INPUT . .true. if all cells in LEXS are .true.  - LOGICAL
!     BALANG- INPUT . .true. if equal group counts            - LOGICAL
!                     (design balanced for groups)
!     BALANT- INPUT . .true. if equal treatment weights       - LOGICAL
!                     (design balanced for treatments)
!     FLAG  - INPUT . Blank string used to enhance error      - CHAR
!                     messages (of length > 50)
!     MAXV  - INPUT . Maximum number of columns of array VV   - INTEGER
!     MAXW  - INPUT . Maximum number of columns of array WW   - INTEGER
!     LEXS  - INPUT . A logical G&T matrix                    - LOGICAL
!                     where existing cells are .true.
!     RCOEF - INPUT . Vector of regression coefficients       - REAL*8
!                     (abscissas)
!     NRCO  - INPUT . Length of RCOEF                         - INTEGER
!     TYPE  - OUTPUT. Type of the comparison:                 - INTEGER
!                        1 = W/G1 type    (product type)
!                        2 = V/T1 type    (product type)
!                        3 = V.W  type    (product type)
!                        4 = general type
!     RTYP  - OUTPUT. Type of regression:                     - INTEGER
!                        0 = no regression requested
!                        1 = degree type
!                        2 = cumulative type
!                     (a negative type codes for the residual
!                     comparison).
!                     When interaction of regressions, RTYP
!                     applies to the last one.
!     KDEG  - OUTPUT. Degree of the regression (if any)       - INTEGER
!     NRSAVD- OUTPUT. Number of regression-variables vectors  - INTEGER
!                     saved on scratch unit.
!                     Relevant only if regression and no
!                     interaction and LPMEAN is .true.
!                     (set to 0 otherwise)
!     NREGC - OUTPUT. Actual number of regression coefficients- INTEGER
!                     used
!     INTCP - OUTPUT. .true. if intercept is requested,       - LOGICAL
!                     .false. otherwise.
!                     When intercept is requested, KDEG
!                     returns 0 and RTYP returns 1
!     PROP  - OUTPUT. .true. if proportionality is requested  - LOGICAL
!                     .false. otherwise.
!                     When proportionality is requested, KDEG
!                     returns 1 and RTYP returns 1 or -1
!     NDF1  - OUTPUT. Number of degrees of freedom of the     - INTEGER
!                     comparison
!     ZINTER- OUTPUT. .true. if an interaction explicitely    - LOGICAL
!                     occurred (i.e. the interaction symbol
!                     appears in the formula)
!     IORD  - OUTPUT. Interaction order + 1 (thus 1 if there  - INTEGER
!                     is no interaction), i.e. the number
!                     of non-degenerated between-parts
!                     comparisons (a degenerated one is one
!                     with only 1 part) (set to 1 if all of
!                     them are degenerated).
!     NVP1  - OUTPUT. Number of parts in the GPAR matrix      - INTEGER
!     NWP1  - OUTPUT. Number of parts in the TPAR matrix      - INTEGER
!     NPART - OUTPUT. Number of parts in the G&T partition,   - INTEGER
!                     i.e. the product of the number of terms
!                     of the between-parts comparisons.
!     GPAR  - OUTPUT. G partition-matrix dimensioned (NGRP,2) - INTEGER
!                     GPAR(:,1) contains the G-marginal
!                     partition (unless TYPE is 4) and
!                     GPAR(:,2) was used as a work array.
!     TPAR  - OUTPUT. T partition-matrix dimensioned (NTRT,2) - INTEGER
!                     TPAR(:,1) contains the T-marginal
!                     partition (unless TYPE is 4) and
!                     TPAR(:,2) was used as a work array.
!     PART  - OUTPUT. G&T partition-matrix                    - INTEGER
!                     If explicit interactions occurred,
!                     it only contains the partition-matrix
!                     corresponding to the last interaction
!                     term and not the overall product
!                     partition-matrix.
!     VV    - OUTPUT. G-coefficients matrix (partly used      - REAL*8
!                     as a work array if interactions)
!                     On return, usefull length reduce to
!                     NGRP*NCV (NCV known on return).
!                     Irrelevant if TYPE is 4
!     WW    - OUTPUT. If TYPE is not 4:                       - REAL*8
!                     T-coefficients matrix (partly used
!                     as a work array if interactions)
!                     On return, usefull length reduce to
!                     NTRT*NCW (NCW known on return).
!                     If TYPE is 4:
!                     G&T coefficients-matrix of length
!                     NTRT*NGRP
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     pacint : ncv  ncw
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  GETUVW  GETWGH  IDNTFY  PACAF5  PACDEG  PACREG  PACOVW
!        PROPAR  PROSET  RESCAL
!     External functions called:
!        NONE
!     Fortran  functions called:
!        INDEX  LEN_TRIM  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst2
   USE pacchr
   USE pacint
   USE pacrnk

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: lenf
   INTEGER, INTENT(IN) :: maxv, maxw, nrco
   LOGICAL, INTENT(IN) :: allgt, balang, balant
   CHARACTER(LEN=*), INTENT(IN) :: form, flag
   LOGICAL, INTENT(IN) :: lexs(ntrt,ngrp)
   REAL(KIND=8), INTENT(IN) :: rcoef(nrco)
   INTEGER, INTENT(OUT) :: type, rtyp, kdeg, nrsavd, nregc, ndf1
   INTEGER, INTENT(OUT) :: iord, nvp1, nwp1, npart
   LOGICAL, INTENT(OUT) :: intcp, prop, zinter, lerr
   REAL(KIND=8), INTENT(OUT) :: vv(ngrp,maxv), ww(ntrt,maxw)
   INTEGER, INTENT(OUT) :: gpar(ngrp,2), tpar(ntrt,2), part(ntrt,ngrp)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, ikar, ityp, i1, i1p, j
   INTEGER :: k, kol, kola, kolh, kolz, k1, k1p
   INTEGER :: len0, loci, locv, locw
   INTEGER :: ndf, ngno0, nladd, npar, ntno0, ntypg, ntypt
   INTEGER :: nume1, nume2, nvdf, nvp, nwdf, nwp, regn, typr
   LOGICAL :: allp, arithg, aritht, helm, metr1, newp
   LOGICAL :: pctni, porp, prtrc, rdeg0, regr, resv, resw, sam0
   CHARACTER(LEN=1), PARAMETER :: mark='$'
   CHARACTER(LEN=160) :: errfmt
   INTEGER :: nxp(2)

   REAL(KIND=8), POINTER :: gweight(:), tweight(:), h1(:)
   REAL(KIND=8), TARGET :: pvwgh(ntrt*ngrp), wwgh(ntrt)
                 ! pvwgh is used to store weights both for G (vwgh)
                 ! and for G&T (pwgh) as never used simultaneously

   INCLUDE 'pacalfp.'

   !--------------------------------------------------------------------
   ! Notes
   !
   ! - When there are interactions of regressions, the abscissa
   ! coefficients taken into account for the 1st polynom are the
   ! m first ones in the list of coefficients (m is the number of
   ! parts involved in the 1st regression), then those for the 2nd
   ! polynom are the n succeding ones in the list (n is the number
   ! of parts involved in the 2nd regression), etc.
   !
   ! - For a multiple df comparison (thus product type currently):
   ! Contrasts on G or T set must be orthogonal according to the
   ! fundamental measure (group counts for G set, treatment weights
   ! for T set).
   ! As in a first step contrasts are built on a partition of the
   ! G/T set (thus with possibly aggregated cells) the following
   ! metric must be used, during this step, in order to lead to
   ! correct final orthogonalization. For any part (involving say,
   ! n cells) the corresponding metric term will be:
   ! . when size weights are requested:
   !   n times the arithmetic mean of the weights of the n cells
   !   belonging to the part.
   ! . when equal weights are requested:
   !   n times the harmonic   mean of the weights of the n cells
   !   belonging to the part (this reduces to the 1st case if
   !   weights involved are equal).
   !
   ! - Orthogonalization of contrasts:
   ! Temporary unit metric is used when product of V or W-contrasts
   ! occurs for an unbalanced product partition, and the resulting
   ! contrasts are orthogonalized in a final step (this is a
   ! fundamental feature of the algorithm and not only a time-saving
   ! feature). Exceptions concern regression-type vectors.
   ! As these conditions are not known prior to formula analysis,
   ! the unit metric is here used (except for regression-type
   ! vectors) if an interaction is detected, and thus correction for
   ! proper metric is made at completion of analysis if no V or
   ! W-product occurred.
   !--------------------------------------------------------------------

   lerr   = .false.
   intcp  = .false.
   prop   = .false.
   rtyp   = 0
   nrsavd = 0
   nregc  = 0
   ndf1   = 1
   npart  = 1
   iord   = 0
   ncv    = 1
   ncw    = 1

      ! nladd, nume1, nume2 are associated to error messages printed
      ! with format errfmt, and prtrc is a flag for abscissas printout
      ! when error.
   errfmt = fmt222
   nladd = 1
   nume1 = 0
   nume2 = 0
   prtrc = .false.
   len0  = max(len_trim(form),1)

      ! ntypg flags product of v matrices.
      ! ntypt flags product of w matrices.
      ! loci points to relevant current column in gpar/tpar: 1 if 1st
      ! interaction member, 2 otherwise (thus it's an interaction flag)
      ! locv/locw points to relevant starting column of coefficients in
      ! v/w for the current interaction term processed.
      ! k1p/i1p are indices of the 1st group/treatment with not all
      ! zero coefficients for the comparison (k1/i1 are similar but
      ! restricted to the current interaction term processed).

   rdeg0 = .false.
   nvp1  = 1
   nwp1  = 1
   ntypg = 0
   ntypt = 0
   loci  = 1
   locv  = 1
   locw  = 1
   k1p   = ngrp
   i1p   = ntrt

      ! See how to derive metric terms for orthogonalization
   arithg = balang .or. gpond == 2
   aritht = balant .or. tpond == 2

      ! See if there is any interaction
   zinter = index( form(:lenf), syinte ) > 0

      ! Get relevant pointers for weights used by idntfy, getuvw and rescal
   if ( gpond == 1 ) then
      gweight => vec1
   else
      gweight => ag
   end if
   if ( tpond == 1 ) then
      tweight => vec1
   else
      tweight => at
   end if

   kol = 0

   do   ! Loop on interaction terms
        ! -------------------------

      kol = kol + 1

         ! Scan for regression or Helmert's contrast
      kolh = kol
      call pacdeg( form, lenf, kol, typr, kdeg, helm, pctni, porp )
      regr = typr /= 0
      if ( regr ) then
         rtyp  = typr
         intcp = pctni
         prop  = porp
         if ( pctni .and. typr < 0 ) then
            errfmt = fmt333
            nladd = 2
            goto 10
         end if
      end if
         ! Analyze the "product of between-parts comparison"
      call pacaf5( form, lenf, kol, ikar, gstr, tstr, gmod, tmod,   &
                   lexs, flag, part, npar, allgt, lerr )
      if ( lerr ) return

         ! Check for an empty comparison
      if ( npar <= 0 ) goto 10

         ! Identify the type of the comparison
      call idntfy( part, npar, ngrp, ntrt, gweight, tweight,                  &
                   ityp, ngno0, k1, pvwgh(1:ntrt*ngrp), ntno0, nvp, nwp, i1,  &
                   gpar(:,loci), tpar(:,loci) )
      k1p = min( k1, k1p )
      i1p = min( i1, i1p )

         ! Df's for current interaction term
      nvdf = max( nvp -1, 1 )
      nwdf = max( nwp -1, 1 )
      ndf  = max( npar-1, 1 )

      if ( regr ) then
         nxp(1) = nwp
         nxp(2) = nvp
            ! Set degree to maximum if none was given
         if ( kdeg < 0 ) kdeg = nxp(ityp) - 1
         regn = nxp(ityp)
         if ( ityp > 2 ) then
            errfmt = fmt444
            nladd = 3
            goto 10
         end if
         if ( typr == -2 .and. kdeg+1 == nxp(ityp) ) then
            errfmt = fmt555
            nladd = 2
            goto 10
         end if
         if ( kdeg >= regn ) then
            errfmt = fmt666
            nladd = 2
            nume1 = regn
            goto 10
         end if
         if ( nrco < nregc+regn ) then
            errfmt = fmt777
            prtrc = .true.
            nladd = 3 + (nregc+regn-1) / 5 + 1
            nume1 = nrco
            nume2 = nregc + regn
            goto 10
         end if
         rdeg0 = kdeg == 0
      end if

      if ( ityp == 4 ) then   ! General type

            ! Currently, no interaction allowed when type is 4
         if ( ikar == kinte .or. loci > 1 ) then
!!          call chkpg( 3, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,3333)
            write(lgu,*) ermrk2
!!          lines = lines + 3
            lerr = .true.
            return
         end if
            ! Currently, only 1 df allowed when type is 4
         if ( npar > 2 ) then
!!          call chkpg( 4, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,4444) npar-1
            write(lgu,*) ermrk2
!!          lines = lines + 4
            lerr = .true.
            return
         end if

      else                    ! Product type

            ! Check for enough room in vv and ww
         if ( locv+nvdf-1 > maxv .or. locw+nwdf-1 > maxw ) then
!!          call chkpg( 2, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,111) form(1:min(len0,72))
            write(lgu,2222)
            write(lgu,*) ermrk2
!!          lines = lines + 2
            lerr = .true.
            return
         end if

         if ( ityp == 3 ) then
            ntypg = ntypg + 1
            ntypt = ntypt + 1
         end if

         if ( ityp == 2 ) then
            ntypg = ntypg + 1
            metr1 = (.not. regr) .and. ( zinter .or. nvdf == 1 .or. (balang .and. nvp == ngno0) )
            if ( metr1 ) then
               h1 => vec1
            else
               h1 => pvwgh
                  ! Fill in pvwgh(1:ngrp) with the relevant metric
               call getwgh( gpar(:,loci), ag, ngrp, nvp, k1,                    &
                            arithg .and. .not. (regr.and.zinter), pvwgh(1:ngrp) )
            end if

         else if ( ityp == 1 ) then
            ntypt = ntypt + 1
            metr1 = (.not. regr) .and. ( zinter .or. nwdf == 1 .or. (balant .and. nwp == ntno0) )
            if ( metr1 ) then
               h1 => vec1
            else
               h1 => wwgh
                  ! Fill in wwgh with the relevant metric
               call getwgh( tpar(:,loci), at, ntrt, nwp, i1,           &
                            aritht .and. .not. (regr.and.zinter), wwgh )
            end if
         end if

      end if

         ! Get contrasts
      if ( regr ) then
            ! Regression type contrasts
            ! h1 points to (inverse of) metric to be used
         if ( ityp == 1 ) then   ! ityp is 1
            j = maxw - locw + 1
            call pacreg( kdeg, typr, pctni, porp, ntrt, j, ngrp, nwp,  &
                         rcoef(1+nregc:),                              &
                         tpar(:,loci), gpar(:,loci), h1,               &
                         ww(:,locw:locw+j-1), vv(:,locv), ndf, nrsavd )
            nwdf = ndf
         else                    ! ityp is 2
            j = maxv - locv + 1
            call pacreg( kdeg, typr, pctni, porp, ngrp, j, ntrt, nvp,  &
                         rcoef(1+nregc:),                              &
                         gpar(:,loci), tpar(:,loci), h1,               &
                         vv(:,locv:locv+j-1), ww(:,locw), ndf, nrsavd )
            nvdf = ndf
         end if
         nregc = nregc + regn
      else
            ! Cancel Helmert type request if not consistent
         if ( helm .and. npar /= 2 ) then
            helm = .false.
            call chkpg( 4, .false., newp )
            write(imp,5555) form(kolh:min(len0,kolh+71))
            lines = lines + 4
            write(lgu,*) ermrk1//ermrk1
            write(lgu,5555) form(kolh:min(len0,kolh+71))
            write(lgu,*) ermrk2//ermrk2
         end if
            ! Get Helmert or "reverse Helmert" type contrasts
         call getuvw( helm, metr1, ityp, ngrp, ntrt, nvp, nwp,      &
                      gpar(:,loci), tpar(:,loci), part,             &
                      gweight, tweight, pvwgh(1:nvp), wwgh(1:nwp),  &
                      vv(:,locv:maxv), maxv-locv+1,                 &
                      ww(:,locw:maxw), maxw-locw+1 )
      end if

         ! If interactions, take products
      if ( loci > 1 ) then
         if ( rdeg0 ) then
            errfmt = fmt888
            nladd = 2
            goto 10
         end if
            ! Take product of marginal partitions
         call propar( gpar(:,1), gpar(:,2), ngrp, nvp1, nvp, ngno0, sam0, allp )
         if ( .not. allp ) goto 20
         call propar( tpar(:,1), tpar(:,2), ntrt, nwp1, nwp, ntno0, sam0, allp )
         if ( .not. allp ) goto 20
            ! Identify type of product
         if ( ityp /= type ) ityp = 3
            ! Take product of contrast vectors
         call proset( vv(:,1:ncv*nvdf), vv(:,locv:locv+nvdf-1), ngrp, ncv, nvdf )
         call proset( ww(:,1:ncw*nwdf), ww(:,locw:locw+nwdf-1), ntrt, ncw, nwdf )
      end if

         ! Restore if intercept option (because no adjusted means are available)
      if ( pctni ) rtyp = 0

      type  = ityp
      npart = npart * npar
      ndf1  = ndf1  * ndf
      ncv   = ncv   * nvdf
      ncw   = ncw   * nwdf
         ! Update iord if the comparison is not degenerated
      if ( npar > 1 ) iord = iord + 1

         ! Analyze next interaction term if any
      loci = 2
      locv = ncv + 1
      locw = ncw + 1
      nvp1 = nvp1 * nvp
      nwp1 = nwp1 * nwp
      if ( ikar /= kinte ) exit
   end do

   ! End of scanning, check end of formula has been reached
   ! ------------------------------------------------------
   if ( ikar /= kblan ) then
      errfmt = fmt999
      goto 10
   end if

      ! Reset iord if the comparison is totally degenerated
   if ( iord <= 0 ) iord = 1

      ! Regression vectors cannot be used to compute adjusted means
      ! if interactions, so consider they were not saved
   if ( iord > 1 ) nrsavd = 0

   ! Orthogonalize and re-norm contrasts if required
   ! -----------------------------------------------
   if ( zinter ) then
         ! As only 1 df is authorized if type is 4, orthogonalization
         ! is irrelevant in this case
      if ( ncw > 1 .and. (type == 1 .or. type == 3) )               &
         call pacovw( ww(:,1:ncw), tpar(:,1), at, ntrt, ncw, nwp1,  &
                      ntno0, i1p, ntypt, balant, aritht, lerr )
      if ( ncv > 1 .and. (type == 2 .or. type == 3) )               &
         call pacovw( vv(:,1:ncv), gpar(:,1), ag, ngrp, ncv, nvp1,  &
                      ngno0, k1p, ntypg, balang, arithg, lerr )
      if ( lerr ) return
   end if

   ! Rescale contrast-coefficients on G&T set
   ! ----------------------------------------
   resv = (type == 2 .or. type == 3) .and. ngno0 > nvp1
   resw = (type == 1 .or. type == 3) .and. ntno0 > nwp1
      ! First update marginal part-weights
   if ( resv ) then   ! Fill in pvwgh with relevant part-weights
      pvwgh(1:nvp1) = zero
      do k = k1p, ngrp
         j = gpar(k,1)
         if ( j > 0 ) pvwgh(j) = pvwgh(j) + gweight(k)
      end do
   end if
   if ( resw ) then   ! Fill in wwgh with t-part-weights
      wwgh(1:nwp1) = zero
      do i = i1p, ntrt
         j = tpar(i,1)
         if ( j > 0 ) wwgh(j) = wwgh(j) + tweight(i)
      end do
   end if
   call rescal( type, ngrp, ntrt, ncv, ncw, gweight, tweight, k1p, i1p, resv, resw,        &
                gpar(:,1), tpar(:,1), pvwgh(1:ngrp), wwgh, vv, maxv, ww, maxw, part, pvwgh )

   ! Normal return
   ! -------------
   return

   ! Error return
   ! ------------
   10 continue
   kola = max( kol-25, 1 )
   kolz = min( kol+25, len0 )
!! call chkpg( 4+nladd, .false., newp )
   write(lgu,*) ermrk1
   write(lgu,111) form(kola:kolz), flag(1:kol-kola+1), mark
   write(lgu,errfmt) nume1, nume2
   if ( prtrc ) write(lgu,'(1x,5g14.7)') rcoef(1:nrco)
   write(lgu,*) ermrk2
!! lines = lines + 4 + nladd
   lerr = .true.
   return

   20 continue
      ! An empty part was found: the comparison is not of interaction type
!! call chkpg( 6, .false., newp )
   write(lgu,*) ermrk1
   write(lgu,111) form(1:min(len0,72))
   write(lgu,1111)
   write(lgu,*) ermrk2
!! lines = lines + 6
   lerr = .true.

   INCLUDE 'pacalff.'

END SUBROUTINE pacalf
