SUBROUTINE pacanv( nvar, nw, dfc, lschef, iuss )

!***********************************************************************
!     - pacanv -
!     performs computations for the standard univariate ANoVa table
!     (equal variances assumed), and prints the table if requested
!***********************************************************************
!
!     NVAR  - INPUT . Number of dependent variables           - INTEGER
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     DFC   - INPUT . Vector of departure from circularity    - REAL*8
!                     for the NVAR dependent variables
!                     (only used if NW > 1)
!     LSCHEF- INPUT . .true. if Scheffe's method for multiple - LOGICAL
!                     comparisons is to be applied
!     IUSS  - INPUT . Unit number where ANOVA information was - INTEGER
!                     stored
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG
!     External functions called:
!        BETACDF
!     Fortran  functions called:
!        LEN_TRIM  MIN  NINT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacchr
   USE pacdtr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nvar, nw, iuss
   LOGICAL, INTENT(IN) :: lschef
   REAL(KIND=8), INTENT(IN) :: dfc(nvar)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: betacdf

   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8
   REAL(KIND=8) :: cm1, cm2, df1, df1a, df2, df2a, f
   REAL(KIND=8) :: proa, prob, sc1, sc2, thf, wx
   INTEGER :: id1, id2, ierr, j, l, lcmpt, ls
   LOGICAL :: adjdf, newp
   CHARACTER(LEN=5), PARAMETER :: gg='(GG):', hf='(HF):'
   CHARACTER(LEN=nocw) :: name
   CHARACTER(LEN=8) :: txf, txp
   CHARACTER(LEN=2) :: mapp(0:3)=(/ '  ', ' a', ' A', ' A' /)

   !--------------------------------------------------------------------
      ! Code for correction for departure from circularity is expanded
      ! rather than forming an isolated procedure, for sake of speed

   lcmpt = min( lrecp-1, len_trim(cmptit) )
   adjdf = nw > 1
   if ( lschef ) then
      ls = 1
   else
      ls = 0
   end if
   if ( adjdf ) then
      wx = nw
      l  = 2
      ls = 3 * ls
   else
      l  = 0
   end if

   if ( lanova ) then
      call chkpg( 5+l+ls, .false., newp )
      if ( newp ) then
         write(imp,555) cmptit(:lcmpt)
         write(imp,555) '  '
         lines = lines + 2
      end if
      write(imp,111)
      lines = lines + 2
   end if

   rewind iuss
   do j = 1, nvar

      read(iuss) name, cm1, cm2, df1, df2, f, prob, iappr
      sc2 = cm2 * df2
      sc1 = cm1 * df1
      id1 = nint( df1 )
      id2 = nint( df2 )

      if ( lanova ) then
         call chkpg( 3+l+ls, .false., newp )
         if ( newp ) then
            write(imp,555) cmptit(:lcmpt)
            write(imp,555) '  '
            write(imp,111)
            lines = lines + 3
         end if
         if ( cm2 > zero ) then
            txf = ' '
            txp = ' '
            write(txf,'(f8.2)') f
            write(txp,'(f8.4)') prob
         else
            txf = '     /  '
            txp = '     /  '
         end if
         write(imp,222) name, sc1, id1, cm1, txf, txp, mapp(iappr), sc2, id2, cm2
         lines = lines + 3
      end if
      if ( lbayes ) then   ! Save info on Bayes-save-file
         write(ibayes,666) "%anova ", name, sc1, df1, cm1, sc2, df2, cm2, f, prob, iappr
      end if
      if ( adjdf .and. cm2 > zero ) then
            ! Greenhouse-Geisser adjustment for df's
         df1a = df1 * dfc(j)
         df2a = df2 * dfc(j)
         cm1  = df1a*f / (df1a*f + df2a)
         proa = betacdf( cm1, half*df1a, half*df2a, ierr )
         if ( ierr == 0 ) then
            proa = one - proa
         else
            proa = -one
         end if
         if ( lanova ) then
            write(imp,333) gg, df1a, df2a, proa, mapp(2)
            lines = lines + 1
         end if
         if ( lbayes ) then   ! Save info on Bayes-save-file
            write(ibayes,777) "%gg ", name, df1a, df2a, proa, 2
         end if
            ! Huynh-Feldt adjustment for df's
         thf = ((df2+wx)*dfc(j)-two)/(df2-wx*wx*dfc(j))
         if ( thf < one ) then
            df1a = df1 * thf
            df2a = df2 * thf
            cm1  = df1a*f / (df1a*f + df2a)
            proa = betacdf( cm1, half*df1a, half*df2a, ierr )
            if ( ierr == 0 ) then
               proa = one - proa
            else
               proa = -one
            end if
         else
            df1a = df1
            df2a = df2
            proa = prob
         end if
         if ( lanova ) then
            write(imp,333) hf, df1a, df2a, proa, mapp(2)
            lines = lines + 1
         end if
         if ( lbayes ) then   ! Save info on Bayes-save-file
            write(ibayes,777) "%hf ", name, df1a, df2a, proa, 2
         end if
      end if

         ! Scheffe's method for multiple comparisons
      if ( lschef .and. cm2 > zero ) then
         df1  = ngrp - 1
         f    = sc1 / (df1*cm2)
         df1  = df1 * nw
         id1  = nint( df1 )
         cm1  = df1*f / (df1*f + df2)
         prob = betacdf( cm1, half*df1, half*df2, ierr )
         if ( ierr == 0 ) then
            prob = one - prob
         else
            prob = -one
         end if
         if ( lanova ) then
            write(imp,444) id1, id2, f, prob, mapp(iappr)
            lines = lines + 1
         end if
         if ( lbayes ) then   ! Save info on Bayes-save-file
            write(ibayes,888) "%#scheffe ", name, df1, df2, f, prob, iappr
         end if
         if ( adjdf ) then
               ! Greenhouse-Geisser adjustment for df's
            df1a = df1 * dfc(j)
            df2a = df2 * dfc(j)
            cm1  = df1a*f / (df1a*f + df2a)
            proa = betacdf( cm1, half*df1a, half*df2a, ierr )
            if ( ierr == 0 ) then
               proa = one - proa
            else
               proa = -one
            end if
            if ( lanova ) then
               write(imp,333) gg, df1a, df2a, proa, mapp(2)
               lines = lines + 1
            end if
            if ( lbayes ) then   ! Save info on Bayes-save-file
               write(ibayes,777) "%#gg ", name, df1a, df2a, proa, 2
            end if
               ! Huynh-Feldt adjustment for df's
!!!!        thf = ((df2+wx)*dfc(j)-two)/(df2-wx*wx*dfc(j))
            if ( thf < one ) then
               df1a = df1 * thf
               df2a = df2 * thf
               cm1  = df1a*f / (df1a*f + df2a)
               proa = betacdf( cm1, half*df1a, half*df2a, ierr )
               if ( ierr == 0 ) then
                  proa = one - proa
               else
                  proa = -one
               end if
            else
               df1a  = df1
               df2a  = df2
               proa  = prob
            end if
            if ( lanova ) then
               write(imp,333) hf, df1a, df2a, proa, mapp(2)
               lines = lines + 1
            end if
            if ( lbayes ) then   ! Save info on Bayes-save-file
               write(ibayes,777) "%#hf ", name, df1a, df2a, proa, 2
            end if
         end if
      end if

   end do

   INCLUDE 'pacanvf.'

END SUBROUTINE pacanv
