SUBROUTINE pacdfc( cov, nw, nvar, nomv, boik, dfc )

!***********************************************************************
!     - pacdfc -
!     calculates and prints a symmetric matrix of indices of Departure
!     From Circularity assumption for a given covariance matrix
!***********************************************************************
!
!     COV   - INPUT . Covariance matrix of order NW*NVAR,     - REAL*8
!                     stored in triangular form
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     NVAR  - INPUT . Number of dependent variables           - INTEGER
!     NOMV  - INPUT . Names  of dependent variables           - CHAR
!     BOIK  - OUTPUT. Boik's multivariate epsilon (1991)      - REAL*8
!                     irrelevant and set to zero if NVAR = 1
!                     -1.0 if error
!     DFC   - OUTPUT. The indices of departure from           - REAL*8
!                     circularity for the NVAR variables
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  GETDFC  MPOWER  MRSZ  MSKIZ  MSZR  PRTSYM
!     External functions called:
!        NONE
!     Fortran  functions called:
!        LEN
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: nw, nvar
   REAL(KIND=8), INTENT(IN) :: cov(nw*nvar*(nw*nvar+1)/2)
   CHARACTER(LEN=*), INTENT(IN) :: nomv(nvar)
   REAL(KIND=8), INTENT(OUT) :: boik
   REAL(KIND=8), INTENT(OUT) :: dfc(nvar)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8
   REAL(KIND=8), PARAMETER :: epsl=1.0e-12_8
      ! epsl is a small real constant used to compute zero tolerance
      ! for eigenvalues as epsl*trace(matrix)/order(matrix)
   REAL(KIND=8) :: tolz, trk, wdbl
   INTEGER :: i, ierr, ii, j, jj, jjj, jw, k
   LOGICAL :: newp
   REAL(KIND=8) :: temp(1)
   REAL(KIND=8) :: wrk(nw*nvar*(nw*nvar+1)+nw*nvar*nw*nvar)

   INCLUDE 'pacdfcp.'

   !--------------------------------------------------------------------

   ! Univariate epsilons
   ! -------------------
   boik = zero

      ! Calculate the matrix of indices of departure (dfc)
   k = nvar*(nvar+1)/2
   call getdfc( cov, nw, nvar, wrk(1:k) )

      ! Print the matrix of indices
   i = imp
   call prtsym( i, .true., wrk(1:k), nvar, nomv, .false., text, len(text) )

      ! Put diagonal elements of wrk in dfc
   dfc(1) = wrk(1)
   if ( nvar == 1 ) return   ! That's all
   j = 1
   do i = 2, nvar
      j = j + i
      dfc(i) = wrk(j)
   end do

   ! Multivariate epsilon (Boik, 1991)
   ! ---------------------------------
      ! Calculate the traces/w of the symmetric sub-matrices nw*nw
      ! in cov, for each pair of variables
      ! The resultant symmetric matrix of order nvar is stored in
      ! wrk(1:) in triangular form
   wdbl = nw
   tolz = zero
   jjj  = 0
   jw   = 0
   do j = 1, nvar
      jj = jw*(jw+1)/2
      do k = 1, j
         ii  = jj - (j-k)*nw
         trk = zero
         do i = jw+1, jw+nw
            ii  = ii + i
            trk = trk + cov(ii)
         end do
         jjj = jjj + 1
         wrk(jjj) = trk / wdbl
      end do
      tolz = tolz + trk
      jw   = jw + nw
   end do
   tolz = epsl * tolz / nvar

   call chkpg( 2, .false., newp )

      ! Calculate inverse of square-root of wrk(1:), resultant
      ! symmetric matrix stored in triangular form in wrk(1:)
   j = nvar*(nvar+1)/2
   call mpower( wrk(1:j),-half, nvar, tolz, ierr )
   if ( ierr /= 0 ) then
      write(imp,111) ierr
      lines = lines + 2
      boik = -one
      return
   end if

      ! Calculate (wrk(1:)oiw).cov.(wrk(1:)oiw) where "o" represents
      ! Kroneker product; resultant symmetric matrix of order nw*nvar
      ! stored in triangular form in wrk(1:)
   ii = nw * nvar
   j  = ii*(ii+1)/2 + 1
   jj = j + j - 1
   call mskiz( wrk(1:nvar*(nvar+1)/2), wrk(j:jj-1), nvar, nw )
   call mszr ( wrk(j:jj-1), cov, wrk(jj:jj+ii*ii-1), ii )
   call mrsz ( wrk(jj:jj+ii*ii-1), wrk(j:jj-1), wrk(1:j-1), ii )

      ! Calculate Boik's epsilon
   call getdfc( wrk(1:), ii, 1, temp )
   boik = temp(1)
!! write(imp,222) boik
!! lines = lines + 2

   INCLUDE 'pacdfcf.'

END SUBROUTINE pacdfc
