SUBROUTINE pacdpa( ius, jus, iupr, xbar, gnum, gapp, gsup, supg,      &
                   ag, hyp0, uw, v, ybar, yqg, ys2, eff, ene, qne,    &
                   nne, qcst, racst, udpw, nudpw, nvar, nsuj, nssav,  &
                   ldptmp, ldpcov, samew, l1df1, noequ )
!***********************************************************************
!     - pacdpa -
!     Derives relevant Protocol (referred to as "y"), stored on ideriv
!     or jus if required, and computes some statistics for that protocol
!     (sum of products, contrast-effects, univariate test-statistics,
!     scale indices...)
!***********************************************************************
!
!     IUS   - INPUT . Unit number for data reading            - INTEGER
!     JUS   - INPUT . Unit number for temporary storage of    - INTEGER
!                     derived protocol
!                     Used only if LDPTMP is .true.
!     IUPR  - INPUT . Unit number for storage of results from - INTEGER
!                     previous comparison
!                     Used only if SAMEW is .true.
!                     Note that rewind must not be done here
!                     as managed by directing routine
!     XBAR  - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     GNUM  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GAPP  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     GSUP  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     SUPG  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     AG    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     HYP0  - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     UW    - INPUT . W or U matrix, depending on whether     - REAL*8
!                     the contrast is product form
!     V     - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     YBAR  - OUTPUT.    see definition of MEM2 sub-arrays    - REAL*8
!     to QNE
!     NNE   - INPUT . Actual length of ENE and QNE            - INTEGER
!     QCST  - INPUT . Sum over G-support groups of (count-1)  - REAL*8
!     RACST - INPUT . Weighting of individual protocols when  - REAL*8
!                     product form and no covariates
!     UDPW  - INPUT . Vector of individual protocol weights   - REAL*8
!                     when non product form (so no covariates)
!                     of length NGRP (only NGSUP usefull) or 1
!     NUDPW - INPUT . Actual length of UDPW                   - INTEGER
!                     (NGRP or 1 if not used)
!     NVAR  - INPUT . Number of variables used                - INTEGER
!     NSUJ  - INPUT . Number of cases used on input           - INTEGER
!     NSSAV - OUTPUT. Number of cases saved                   - INTEGER
!     LDPTMP- INPUT . .true. if derived protocol saving       - LOGICAL
!                     requires temporary storage on JUS
!     LDPCOV- INPUT . .true. if derived protocol saving       - LOGICAL
!                     cannot be completed here
!                     (it implies LDPTMP)
!     SAMEW - INPUT . .true. if results from the previous     - LOGICAL
!                     comparison (stored on IUPR) can be used
!     L1DF1 - INPUT . .true. if the comparison has 1 df       - LOGICAL
!     NOEQU - INPUT . .true. if univariate test not assuming  - LOGICAL
!                     equal variances can be computed
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ABS  SQRT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE pacchr
   USE pacint
   USE paclgl
   USE spcval

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: qcst, racst
   INTEGER, INTENT(IN) :: ius, jus, iupr, nne, nudpw, nvar, nsuj
   LOGICAL, INTENT(IN) :: ldptmp, ldpcov, samew, l1df1, noequ
   REAL(KIND=8), INTENT(IN) :: xbar(nlxb,ngrp), ag(ngrp), hyp0(nvar)
   REAL(KIND=8), INTENT(IN) :: uw(ntrt,nszu), v(ngrp,ncv)
   REAL(KIND=8), INTENT(IN) :: udpw(nudpw)
   INTEGER, INTENT(IN) :: gnum(ngrp), gapp(ngrp), gsup(ngrp), supg(ngsup)
   INTEGER, INTENT(OUT) :: nssav
   REAL(KIND=8), INTENT(OUT) :: ybar(nwdc,ngsup)
   REAL(KIND=8), INTENT(OUT) :: yqg(nwdctg), ys2(nwdct)
   REAL(KIND=8), INTENT(OUT) :: eff(ncv,nwdc), ene(nne), qne(nne)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, two=2.0_8, three=3.0_8, four=4.0_8, five=5.0_8
   REAL(KIND=8) :: e, frq, ragm5, rk, rqcst, sab, sumx, vvaa
   INTEGER :: i, ij, ik, ix, j, jt, k, kase, kdum, kg, kjl, ko, ks, kx
   INTEGER :: l, ldum, lj, ljk, ljt, lljj, m, num, ny
   LOGICAL :: ldpsav
   CHARACTER(LEN=incdw) :: slab
   REAL(KIND=8) :: xg(ngfac), x(ntdc), y(nwdc)
                 ! xg - work vector for grouping variables of case read
                 ! x  - work vector for x-data of case read
                 ! y  - work vector for the derived data of case read

   !--------------------------------------------------------------------
   !  Note.
   !  sab is used to compute a rough approximation of the accuracy
   !  of some measures. If zero is within the accuracy range then
   !  the measure is reset to zero. This is done to avoid some true
   !  zeroes replaced by extremely small values because of the
   !  machine precision.
   !  For this purpose it is assumed the relative errors on uw and x
   !  are equal to zerolk, and only a lower bound of the errors is
   !  calculated. Note also that the calculation of error may be
   !  done on some terms of a summation and not on the overall sum
   !  (e.g. on individual deviations from mean but not on the
   !  variance)
   !--------------------------------------------------------------------

      ! See if derived protocol is to be saved here
   ldpsav = ( lderiv .or. listdp ) .and. ( .not. lrduce .or. nwdc == 1 )
      ! Get appropriate length of y when derived protocol is saved
      ! (if uw is such that derived covariates, if any, are always zero
      ! then they are not saved)
   ny = nwdc
   if ( .not. ldpcov ) ny = ncw * ndep

      ! Prepare constants used to get relevant derivation coefficients:
      ! If the contrast is product form then uw is actually array
      ! w(ntrt,ncw), otherwise it's array u(ntrt,ngrp)
      ! kdum codes whether w (then 0) or u (then 1), ldum being its
      ! complementary to 1
      ! (they are used to get suitable index in uw during derivation)
   if ( lupvw ) then
      kdum = 0
   else
      kdum = 1
   end if
   ldum = 1 - kdum

      ! Initialize ene and qne if they are to be used
   if ( l1df1 ) then
      ene(1:nvar) = zero
      qne(1:nvar) = zero
   end if

   ! Compute y-means, effects and initialize ene, qne
   ! ------------------------------------------------

   ij = -ntrt
   lj = 0
   do j = 1, nvar
      ij = ij + ntrt
      do l = 1, ncw
         lj = lj + 1
         do m = 1, ncv
            e = zero
            do ks = 1, ngsup
               k = supg(ks)
               ix = k*kdum + l*ldum
               kx = gapp(k)
               sumx = zero
               sab = zero
               do i = 1, ntrt
                  sumx = sumx + uw(i,ix)*xbar(i+ij,kx)
                  sab = sab + abs( uw(i,ix)*xbar(i+ij,kx) )
               end do
               if ( abs(sumx) <= two*sab*zerolk ) sumx = zero
               ybar(lj,ks) = sumx
               e = e + v(k,m)*sumx
            end do
            eff(m,lj) = e
         end do
      end do
   end do

   ! Derive relevant data and compute yqg and ys2
   ! --------------------------------------------

   if ( samew ) then
         ! Use matrices from previous comparison
         ! Rewind must not be done here as managed by directing routine
      read(iupr) ys2
      read(iupr) yqg
         ! If derived protocol is not to be saved, no use reading data
      if ( .not. ldpsav ) goto 10
   else
         ! Initialize ys2 and yqg
      ys2(1:nwdct) = zero
      yqg(1:nwdct) = zero
      yqg(nwdct+1:nwdctg) = zero
   end if

      ! Remember that ideriv was rewinded in pacder if relevant
   if ( ldptmp ) rewind jus
   rewind ius
   nssav = 0

   do kase = 1, nsuj
         ! num is sequential case-number in original input data file
         ! frq is case frequency
         ! ko  is group-number according to appearance order in data
         ! k   is group-number according to "actual group-table"
         ! ks  is group-number according to "G-support group-table"
      read(ius) num, slab, frq, ko, xg, x
      k = gnum(ko)
      ks = gsup(k)
         ! Skip the case if it doesn't belong to a G-support group
      if ( ks <= 0 ) cycle

      ik = k * kdum
      kg = nwdct * (ks-1)
      ij = -ntrt
      lj = 0
      ljt = 0

      do j = 1, nvar
         ij = ij + ntrt

         do l = 1, ncw
            ix = ik + l*ldum
            ljt = ljt + lj
            lj = lj + 1

               ! Compute relevant data for this variable
            sumx = zero
            sab = zero
            do i = 1, ntrt
               sumx = sumx + uw(i,ix)*x(i+ij)
               sab = sab + abs( uw(i,ix)*x(i+ij) )
            end do
            if ( abs(sumx) <= two*sab*zerolk ) sumx = zero
            y(lj) = sumx
               ! Update sub-matrix of sum-of-products on W*R
               ! (Note that no provision is made for non product form
               ! when computing ys2, thus it's irrelevant in this case)
            if ( .not. samew .and. frq > zero ) then
               sumx = sumx - ybar(lj,ks)
               if ( abs(sumx) <= zerolk*(two*abs(y(lj))+three*abs(ybar(lj,ks))) ) sumx = zero
               do lljj = 1, lj
                  kjl = lljj + ljt
                  e = sumx * (y(lljj)-ybar(lljj,ks)) * frq
                  ys2(kjl) = ys2(kjl) + e
                  ljk = kjl + kg
                  yqg(ljk) = yqg(ljk) + e
               end do
            end if
         end do

      end do

         ! If required, save relevant data of current case
      if ( ldpsav ) then
         nssav = nssav + 1
         if ( lupvw ) then
            sumx = racst
         else
            sumx = udpw(ks)
         end if
         if ( ldpcov ) then        ! Temporary storage (full Y saved as needed further)
            write(jus) slab, frq, sumx, k, xg, y
         else if ( ldptmp ) then   ! Temporary storage (only dep. var. are saved)
            write(jus) slab, frq, sumx, k, xg, y(1:ny)
         else if ( lderiv ) then   ! Save on user's save-file (only dep. var. are saved)
            rk = k
            write(ideriv,derfmt) slab, frq, sumx, rk, xg, y(1:ny)
         end if
      end if

   end do

      ! Complete calculation of ys2
   if ( .not. samew .and. qcst > zero ) then
      rqcst = one / qcst
      do lj = 1, nwdct
         ys2(lj) = ys2(lj) * rqcst
      end do
   end if


10 continue

   ! If one df, rescale effect and, not assuming equal variances,
   ! ------------------------------------------------------------
   ! compute adjusted scale indices and q coefficients
   ! -------------------------------------------------

   if ( l1df1 ) then

      eff(1,1:nvar) = eff(1,1:nvar) - hyp0(1:nvar)

      if ( noequ ) then

         kg = 0
         do ks = 1, ngsup
            k = supg(ks)
            vvaa = (v(k,1)*v(k,1)) / (ag(k)*(ag(k)-three))
            ragm5 = one / (ag(k)-five)

            jt = 0
            do j = 1, nvar
               jt = jt + j
               sumx = vvaa * yqg(kg+jt)
               ene(j) = ene(j) + sumx
               qne(j) = qne(j) + (sumx*sumx*ragm5)
            end do

            kg = kg + nwdct
         end do

         where ( qne > xminim )
            qne = four + (ene*ene/qne)
            ene = sqrt( (qne-two)*ene/qne )
         elsewhere
            qne = zero
            ene = zero
         end where

      end if

   end if

END SUBROUTINE pacdpa
