SUBROUTINE pacfeq( bcor, cstb, bmat, lbmat, ys2, rep, indv, nr, nw,  &
                   efmag, stdv, f, e )
!***********************************************************************
!     - pacfeq -
!     computes univariate statistics (F) for the indv-th variable under
!     EQual variances and circularity assumptions
!***********************************************************************
!
!     BCOR  - INPUT . .true. if the corrective matrix BMAT is - LOGICAL
!                     to be used (i.e. covariates are used
!                     and NR > 1)
!     CSTB  - INPUT . A coefficient involved in computation   - REAL*8
!                     of scale index of Bayes distribution
!                     It is the square of component of norm
!                     of G-contrast vectors (or 1.0 when the
!                     procedure applies to slopes of dep. var.
!                     for covariates) (May be multiplied by a
!                     constant if covariates)
!                     Note: if adjustment by covariates and
!                     NR is 1, then it was divided by BMAT(1)
!                     Used only if comparison is product form
!     BMAT  - INPUT . A corrective symmetric matrix, of order - REAL*8
!                     NR, stored in triangular form (arising
!                     from adjustment by covariates)
!                     Used only if BCOR is .true.
!     LBMAT - INPUT . Actual dimension of BMAT                - INTEGER
!                     (NR*(NR+1)/2 or 1 if not used)
!     YS2   - INPUT . Within-groups covariance matrix, for    - REAL*8
!                     all variables, of order NW*(nb of var.),
!                     stored in triangular form
!     REP   - INPUT . Representant of effect, with NR rows    - REAL*8
!                     and NW*(nb of variables) columns
!     INDV  - INPUT . Index of the relevant variable          - INTEGER
!     NR    - INPUT . Number of rows in REP                   - INTEGER
!     NW    - INPUT . Number of W-contrasts                   - INTEGER
!     EFMAG - OUTPUT. Numerical indicator of magnitude of the - REAL*8
!                     effect for INDV-th variable
!     STDV  - OUTPUT. Within-groups standard deviation for    - REAL*8
!                     INDV-th variable
!     F     - OUTPUT. F-statistic for INDV-th variable        - REAL*8
!     E     - OUTPUT. Scale index of Bayesian distribution    - REAL*8
!                     for INDV-th variable
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        NONE
!     External functions called:
!        NONE
!     Fortran  functions called:
!        MAX  MIN  SQRT
!
!-----------------------------------------------------------------------

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: cstb
   INTEGER, INTENT(IN) :: lbmat, indv, nr, nw
   LOGICAL, INTENT(IN) :: bcor
   REAL(KIND=8), INTENT(IN) :: bmat(lbmat), ys2(*), rep(nr,*)
   REAL(KIND=8), INTENT(OUT) :: efmag, stdv, f, e

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   REAL(KIND=8) :: tot
   INTEGER :: i, ij, j, jj, k, kn, k1

   !--------------------------------------------------------------------

      ! Assuming rep and ys2 are restricted to the relevant subset for
      ! the given variable, we have:
      ! efmag**2 = trace(rep'*bmat*rep) / (nr*nw)
      !    (if no adjustment, bmat is not used)
      ! stdv**2 = trace(ys2) / nw
      ! e**2 = cstb * stdv
      ! f = efmag / e

   efmag = zero
   e = zero
      ! k1 and kn give the relevant rows in ys2 for indv-th variable
   kn = nw * indv
   k1 = kn - nw + 1
   jj = (k1-1) * k1 / 2

   if ( bcor ) then

      do k = k1, kn
         jj = jj + k
         e = e + ys2(jj)

         do i = 1, nr
            tot = zero
            do j = 1, nr
               ij = max(i,j)
               ij = ij*(ij-1)/2 + min(i,j)
               tot = tot + bmat(ij)*rep(j,k)
            end do
            efmag = efmag + rep(i,k)*tot
         end do

      end do

   else

      do k = k1, kn
         jj = jj + k
         e = e + ys2(jj)

         do i = 1, nr
            efmag = efmag + rep(i,k)*rep(i,k)
         end do

      end do

   end if

   efmag = efmag / (nr*nw)
   stdv = e / nw
   e = cstb * stdv
   if ( e > zero ) then
      f = efmag / e
      stdv = sqrt(stdv)
      e = sqrt(e)
   else
      f = zero
   end if
   efmag = sqrt(efmag)

END SUBROUTINE pacfeq
