SUBROUTINE pacfor( komp, last, oldcnd, noldc, loldc, boldc,            &
                   oldfor, noldf, loldf, rcoef, nrco, balang, balant,  &
                   part, cmptyp, rtyp, kdeg, ndf1, npart, nrsavd, lerr )
!***********************************************************************
!     - pacfor -
!     directs the analysis of a comparison-FORmula.
!     The analysis results in matrices of contrast-coefficients (V,W/U)
!***********************************************************************
!
!     KOMP  - INPUT . Index of formula currently processed    - INTEGER
!     LAST  - INPUT . Length of that formula                  - INTEGER
!     OLDCND- INPUT . Previous condition (if any)             - CHAR
!             OUTPUT. Current one (if any)
!     NOLDC - INPUT . Length of OLDCND                        - INTEGER
!             OUTPUT.
!     LOLDC - INPUT . .true. if a condition was previously    - LOGICAL
!                     specified, .false. otherwise
!             OUTPUT. .true. if .true. on input or a condition
!                     is specified in current formula
!     BOLDC - INPUT . .true. if the matrix corresponding to   - LOGICAL
!                     the previous condition must be rebuilt
!                     (because destroyed by a new design),
!                     .false. otherwise
!     OLDFOR- INPUT . Previous comparison (left part)(if any) - CHAR
!             OUTPUT. Current one (if any)
!     NOLDF - INPUT . Length of OLDFOR                        - INTEGER
!             OUTPUT.
!     LOLDF - INPUT . .true. if a comparison was previously   - LOGICAL
!                     specified, .false. otherwise
!             OUTPUT. .true. unless no comparison was ever
!                     entered
!     RCOEF - INPUT . Vector of regression coefficients       - REAL*8
!     NRCO  - INPUT . Length of RCOEF                         - INTEGER
!     BALANG- INPUT . .true. if equal group counts            - LOGICAL
!                     (design balanced for groups)
!     BALANT- INPUT . .true. if equal treatment weights       - LOGICAL
!                     (design balanced for treatments)
!     PART  - OUTPUT. G&T partition matrix                    - INTEGER
!     CMPTYP- OUTPUT. Type of comparison:                     - INTEGER
!                        1 = W/G1 type    (product form)
!                        2 = V/T1 type    (product form)
!                        3 = V.W  type    (product form)
!                        4 = general type (non-product form)
!     RTYP  - OUTPUT. Type of regression:                     - INTEGER
!                        0 = no regression requested
!                        1 = degree type
!                        2 = cumulative type
!                     (a negative type codes for the residual
!                     comparison)
!     KDEG  - OUTPUT. Degree of the regression (if any)       - INTEGER
!     NDF1  - OUTPUT. Number of degrees of freedom of the     - INTEGER
!                     comparison
!     NPART - OUTPUT. Number of parts of the partition matrix - INTEGER
!                     PART (set to 0 if the matrix is not
!                     available, e.g. when nested comparisons)
!     NRSAVD- OUTPUT. Number of regression-variables vectors  - INTEGER
!                     saved on scratch unit
!                     Relevant only if regression and LPMEAN
!                     is .true. (set to 0 otherwise)
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     carl   : car
!     gmpint : lines
!     memst2 : v  w
!     pacint : ncv  ncw  nszu  gpond  tpond
!     paclgl : lupvw
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  FILOPN  IDNTFY  GMP999  PACAF5  PACALF  PACARF  PACHMC
!        PACLCF  PAC999
!     External functions called:
!        JCHAR  VALRDY
!     Fortran  functions called:
!        ABS  ACHAR  ALL  ALLOCATE  ALLOCATED  DEALLOCATE  INDEX  LEN
!        MAX  MIN  PACK
!
!-----------------------------------------------------------------------

   USE carl
   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst2
   USE pacchr
   USE pacint
   USE paclgl
   USE pacrnk
   USE spcval

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: komp, last, nrco
   LOGICAL, INTENT(IN) :: boldc, balang, balant
   REAL(KIND=8), INTENT(IN) :: rcoef(nrco)
   INTEGER, INTENT(INOUT) :: noldc, noldf
   LOGICAL, INTENT(INOUT) :: loldc, loldf
   CHARACTER(LEN=*), INTENT(INOUT) :: oldcnd, oldfor
   INTEGER, INTENT(OUT) :: cmptyp, rtyp, kdeg, ndf1, npart, nrsavd
   LOGICAL, INTENT(OUT) :: lerr
   INTEGER, INTENT(OUT) :: part(ntrt,ngrp)

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: jchar
   LOGICAL, EXTERNAL :: valrdy

   INTEGER :: i, ik, ikar, inst, iord, iord0, ityp, i1nest
   INTEGER :: k, kol, kola, kolz, k1nest, l1, l2, l3, lvv, lww
   INTEGER :: maxv, maxw, mxv, mxw
   INTEGER :: n, ndf0, nestyp, nexpf, ngno0, ngpar
   INTEGER :: nladd, nparn, npar0, nregc, nst1, nst2, ntno0, ntpar
   INTEGER :: nvdf, nwdf, nvnest, nwnest
   INTEGER :: pr, qw, rp, status, wq, xinte, xnest
   LOGICAL :: gtall, intcp, newp, pend, prop, zinter, zpmean
   CHARACTER(LEN=1024) :: expf
   CHARACTER(LEN=36) :: rmsg
   REAL(KIND=8) :: vwgh(ntrt*ngrp)   ! part-weights
   REAL(KIND=8), ALLOCATABLE :: vv(:,:), ww(:,:)
   REAL(KIND=8), POINTER :: gag(:), tat(:)
   INTEGER :: ndel(2)
   INTEGER :: gpar(ngrp,2), tpar(ntrt,2)
            ! gpar(:,1) is G-partition and gpar(:,2) is a work array in pacalf
            ! tpar(:,1) is T-partition and tpar(:,2) is a work array in pacalf
   INTEGER :: zgpar(ngrp,2), ztpar(ntrt,2), zpart(ntrt,ngrp)
   LOGICAL, POINTER :: lexs(:,:)
   CHARACTER(LEN=2) :: loop(2)
   CHARACTER(LEN=1) :: deli(7)

   INCLUDE 'pacforp.'

      ! msgr are messages associated to a regression type comparison
      ! ntyp are the set(s) referred to by the nesting part (if any)
      ! of the comparison

   !--------------------------------------------------------------------
      ! Note.
      ! On input the dimensions for v and w are respectively:
      ! ngrp * max(ngrp,2)  and  ntrt * max(ntrt,2)
      ! but the relevant room needed on return is:
      ! ngrp * ncv  and  ntrt * ncw  (note that for nested comparisons
      ! ncv and ncw may be greater than ngrp and ntrt)
   !--------------------------------------------------------------------

      ! Copy lpmean as temporarily changed when nested comparisons
      ! and restored if error return
   zpmean = lpmean

      ! Prepare arrays needed by paclcf (loop expansion)
   loop(1) = syor // syor
   loop(2) = sycomp // sycomp
   ndel(1) = 7
   ndel(2) = 5
   deli(1) = sycond
   deli(2) = synest
   deli(3) = syinte
   deli(4) = syprod
   deli(5) = sycomp
   deli(6) = syand
   deli(7) = syor

   ! Get weighting options
   ! ---------------------
      ! gpond refers to groups and tpond to treatments
      ! 1 codes for equal weights, 2 for group-size or treament weights
      ! l1 is location in cmpfor of 1st character of the comparison
      ! l2 is location of last character (condition excluded)
      ! l3 is location of conditioning symbol (= l2 if none)
   l1 = 1
   if ( last > 1 .and. ( cmpfor(komp)(2:2) == optw0 .or. &
                         cmpfor(komp)(3:3) == optw0 ) ) then
      l1 = 3
      wopt = achar( jchar( cmpfor(komp)(1:1) ) )
      if ( wopt == optw2 .or. wopt == optw3 ) then
         gpond = 2
      else if ( wopt == optw1 ) then
         gpond = 1
      else
!!       call chkpg( 3, .false., newp )
         write(lgu,*) ermrk1
         write(lgu,333) cmpfor(komp)(:2), optw1, optw0, optw2, optw0, optw3, optw0
         write(lgu,*) ermrk2
!!       lines = lines + 3
         goto 10
      end if
      if ( cmpfor(komp)(3:3) == optw0 ) then
         l1 = 4
         wopt = achar( jchar( cmpfor(komp)(2:2) ) )
         if ( wopt == optw2 .or. wopt == optw3 ) then
            tpond = 2
         else if ( wopt == optw1 ) then
            tpond = 1
         else
!!          call chkpg( 3, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,333) cmpfor(komp)(:2), optw1, optw0, optw2, optw0, optw3, optw0
            write(lgu,*) ermrk2
!!          lines = lines + 3
            goto 10
         end if
      end if
   end if
   if ( gpond == 1 ) then
      cmptit(1:1) = optw1
   else
      cmptit(1:1) = optw2
   end if
   if ( tpond == 1 ) then
      cmptit(2:3) = optw1 // optw0
   else
      cmptit(2:3) = optw2 // optw0
   end if

   ! See if the comparison is conditioned
   ! ------------------------------------
   l3 = index( cmpfor(komp)(:max(last,1)), sycond )
   l2 = l3 - 1
   if ( l3 <= 0 ) then
      lexs => lgtx   ! No condition, so lgtx will be the initializing matrix in pacalf
      l2 = last
   else
      lexs => lres   ! Condition, so lres will be the initializing matrix in pacalf
   end if

   ! See if a comparison is entered
   ! ------------------------------
   if ( l1 <= l2 ) then
         ! A new comparison is given
      loldf = .true.
      noldf = l2 - l1 + 1
      oldfor = cmpfor(komp)(l1:l2)
      cmptit(4:) = cmpfor(komp)(l1:)
   else if ( loldf ) then
         ! Use previous comparison
      nladd = 2 + (noldf-1)/(lrecp-1)
      call chkpg( nladd, .false., newp )
      write(imp,111) coco(1), (oldfor(i:min(i+lrecp-2,noldf)),i=1,noldf,lrecp-1)
      lines = lines + nladd
      cmptit(4:) = oldfor(:noldf)
   else
         ! Error
!!    call chkpg( 3, .false., newp )
      write(lgu,*) ermrk1
      write(lgu,222) coco(1)
      write(lgu,*) ermrk2
!!    lines = lines + 3
      goto 10
   end if

   ! Look at the condition (if any)
   ! ------------------------------
   if ( l3 > 0 ) then
      if ( l3 < last ) then   ! A new condition is given
            ! Expand loops
         call paclcf( cmpfor(komp)(l3:last), last-l3+1, loop, deli, ndel, expf, nexpf, lerr )
         if ( lerr ) goto 10
            ! Analyze this condition
         call pacarf( expf, nexpf, gstr, tstr, gmod, tmod, lgtx, flag, lexs, lerr )
         if ( lerr ) goto 10
         loldc  = .true.
         noldc  = last - l3 + 1
         oldcnd = cmpfor(komp)(l3:last)
      else                    ! Use previous condition
         if ( loldc ) then
            nladd = 2 + (noldc-1)/(lrecp-1)
            call chkpg( nladd, .false., newp )
            write(imp,111) coco(2), (oldcnd(i:min(i+lrecp-2,noldc)),i=1,noldc,lrecp-1)
            lines = lines + nladd
            if ( boldc ) then
                  ! Expand loops
               call paclcf( oldcnd(:noldc), noldc, loop, deli, ndel, expf, nexpf, lerr )
               if ( lerr ) goto 10
                  ! Build the corresponding logical matrix
               call pacarf( expf, noldc, gstr, tstr, gmod, tmod,  &
                            lgtx, flag, lexs, lerr )
               if ( lerr ) goto 10
            end if
         else
!!          call chkpg( 3, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,222) coco(2)
            write(lgu,*) ermrk2
!!          lines = lines + 3
            goto 10
         end if
      end if
      if ( noldf+4 <= len(cmptit) ) cmptit(noldf+4:) = oldcnd(:noldc)
   end if

   ! See if all cells in the design are used
   ! ---------------------------------------
   gtall = all(pack(lexs,mask=.true.))

      ! Scan for nesting symbol
   nestyp = 0
   xnest  = index( oldfor(:noldf), synest )

   if ( xnest <= 0 ) then

      ! No nesting, analyze left part of formula
      ! ----------------------------------------
         ! This will result in partition matrices and contrast matrices
         ! These matrices will be marginal ones if the comparison is
         ! product-form, and the complete G&T ones otherwise.
         ! In partition matrices (gpar/tpar/part) each cell codes the
         ! class of the partition to which belongs the corresponding
         ! (marginal) cell in the design (0 for cells not involved in
         ! the comparison)

         ! Expand loops
      call paclcf( oldfor, noldf, loop, deli, ndel, expf, nexpf, lerr )
      if ( lerr ) goto 10

         ! Get maximum size of arrays v and w/u
         ! Remember w and u are a same array as they cannot be
         ! simultaneously relevant
         ! Note that non-product form comparisons are currently limited
         ! to 1 df so that maximum size for u is ntrt*ngrp
      maxv = max( ngrp, 2 )
      maxw = max( ntrt, 2 )
      maxw = max( maxw, ngrp )
         ! Analyze (ncv & ncw known at completion of analysis, thus
         ! allocate temporary arrays vv and ww for v and w that will
         ! be allocated when their actual sizes is known)
      if ( allocated(vv) ) deallocate(vv)
      if ( allocated(ww) ) deallocate(ww)
      allocate ( vv(ngrp,maxv), ww(ntrt,maxw), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) ermrk1
         write(lgu,101) " (PacFor)."
         write(lgu,*) ermrk2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if
      call pacalf( expf, nexpf, gtall, balang, balant,      &
                   flag, maxv, maxw, lexs, rcoef, nrco,     &
                   cmptyp, rtyp, kdeg, nrsavd,              &
                   nregc, intcp, prop, ndf1, zinter, iord,  &
                   ngpar, ntpar, npart,                     &
                   gpar, tpar, part, vv, ww, lerr )
      if ( lerr ) goto 10

         ! If the partition associated to the comparison is to be
         ! printed and if there are interactions, then matrix part
         ! is built as the product of the marginal partitions
      if ( lpmean .and. zinter ) then
         do k = 1, ngrp
            if ( gpar(k,1) > 0 ) then
               do i = 1, ntrt
                  if ( tpar(i,1) > 0 ) then
                     part(i,k) = tpar(i,1)  +  (gpar(k,1)-1) * ntpar
                  else
                     part(i,k) = 0
                  end if
               end do
            else
               part(:,k) = 0
            end if
         end do
         npart = ngpar * ntpar
      end if

   else

      ! Nested comparison
      ! -----------------
         ! nst1 is length of nested part of formula
         ! nst2 is length of nesting part of formula
         ! If previous comparison (in oldfor) is used, it is already
         ! written in canonical form
      nst1 = xnest - 1

      if ( l1 <= l2 ) then
            ! A new comparison was entered (now in oldfor),
            ! so it must be re-written in canonical form
            ! Nesting part is temporarily stored in car(1:maxcf),
            ! nested part is left, but compressed, in oldfor

            ! Look for interaction symbol as delimitor
         xinte = index( oldfor(xnest:noldf), syinte ) + xnest - 1

         if ( xinte >= xnest ) then
            pend = .true.
            nst1 = noldf
            pr = 1
            qw = 1
               ! Loop on nesting & interaction symbols as delimitors
               ! of nesting/nested subparts (pend flags looping to
               ! be continued)
            do
               n = xinte - xnest
                  ! Update nesting part
               wq = qw + n - 1
               car(qw:wq) = oldfor(xnest:xinte-1)
               qw = wq + 1
                  ! Compress nested part
               rp = nst1
               nst1 = nst1 - n
               if ( pend ) then
                  car(maxcf+1:maxcf+rp-xinte+1) = oldfor(xinte:rp)
                  oldfor(xnest:nst1) = car(maxcf+1:maxcf+rp-xinte+1)
                  pr = xnest + 1
                     ! Look for next nesting symbol
                  xnest = index( oldfor(pr:nst1), synest ) + pr - 1
                  if ( xnest >= pr ) then
                        ! Look for next interaction symbol
                     xinte = index( oldfor(xnest:nst1), syinte ) + xnest - 1
                     if ( xinte < xnest ) then
                           ! Next pass will be the last one
                        xinte = nst1 + 1
                        pend = .false.
                     end if
                     cycle   ! Repeat (or end) process
                  end if
               end if
               exit
            end do

               ! Copy nesting part in oldfor, after nested part
            oldfor(nst1+1:) = car(:qw-1)
         end if

         cmptit(4:noldf+3) = oldfor(:noldf)

         if ( oldfor(:noldf) /= cmpfor(komp)(l1:l2) ) then
            ik = noldf
            car(:noldf) = oldfor(:noldf)
            if ( l3 > 0 ) then
               ik = ik + noldc
               car(noldf+1:ik) = oldcnd(:noldc)
            end if
            nladd = 2 + (ik-1)/(lrecp-1)
            call chkpg( nladd, .false., newp )
            write(imp,444) (car(i:min(i+lrecp-2,ik)),i=1,ik,lrecp-1)
            lines = lines + nladd
         end if
      end if
         ! Length of nesting part
      nst2 = noldf - nst1

      ! Analyze nesting part
      ! --------------------
         ! Expand loops
      call paclcf( oldfor(nst1+1:), nst2, loop, deli, ndel, expf, nexpf, lerr )
      if ( lerr ) goto 10
         ! Analyze
      kol = 2
      call pacaf5( expf, nexpf, kol, ikar, gstr, tstr, gmod, tmod,  &
                   lexs, flag, part, nparn, gtall, lerr )
      if ( lerr ) goto 10
         ! Check end of formula has been reached
      if ( ikar /= kblan ) then
         kola = max( kol-25, 1 )
         kolz = min( kol+25, last )
!!       call chkpg( 5, .false., newp )
         write(lgu,*) ermrk1
         write(lgu,3333) oldfor(kola:kolz), flag(1:kol-kola+1), mark
         write(lgu,*) ermrk2
!!       lines = lines + 5
         goto 10
      end if
         ! Identify the type of the nesting part
         ! (this is only needed to complete identification of the type
         ! of comparison and thus for printing possible warning on
         ! the validity of some computations, but in no way for the
         ! computations themselve)
      if ( gpond == 1 ) then
         gag => vec1
      else
         gag => ag
      end if
      if ( tpond == 1 ) then
         tat => vec1
      else
         tat => at
      end if
      call idntfy( part, nparn, ngrp, ntrt, gag, tat,     &
                   nestyp, ngno0, k1nest, vwgh(1:nparn),  &
                   ntno0, nvnest, nwnest, i1nest,         &
                   gpar(:,1), tpar(:,1) )
      if ( nestyp > 3 ) nestyp = 3
         ! Save condition and partition matrices as these arrays will
         ! be needed and modified during the analysis of nested part
      call filopn( 0, junit1, ' ', 'SCRATCH', 'UNFORMATTED', .false., lerr )
      if ( lerr ) then
         lpmean = zpmean
         return
      end if
      rewind junit1
      write(junit1) part
      write(junit1) lexs

      ! Analyze nested part
      ! -------------------
         ! N.B. V and W df's may be greater than ngrp and ntrt
      maxv = max( nparn*(ngrp-1)+1, 2 )
      maxw = max( nparn*(ntrt-1)+1, 2 )
      nszu = ngrp
      maxw = max( maxw, nszu )
      if ( allocated(vv) ) deallocate(vv)
      if ( allocated(ww) ) deallocate(ww)
      allocate ( vv(ngrp,maxv), ww(ntrt,maxw), stat=status )
      if ( status /= 0 ) then   ! fatal error
         write(lgu,*) ermrk1
         write(lgu,101) " (PacFor)."
         write(lgu,*) ermrk2
         call gmp999            ! release memory space and files
         call pac999( 99 )
         stop
      end if

         ! Expand loops, so that expf will contain expanded oldfor
      call paclcf( oldfor, nst1, loop, deli, ndel, expf, nexpf, lerr )
      if ( lerr ) goto 10
         ! Expf now contains expanded oldfor

         ! Loop on every nesting part
         ! Reset lpmean to .false. (as not available) to avoid useless
         ! saving of vectors on scratch unit when regression (but don't
         ! forget to restore it at completion of analysis)
         ! Reset gtall as obviously not all cells exist for each pass
      lpmean = .false.
      gtall = .false.
      nvdf = 0
      nwdf = 0
      ndf1 = 0
      lvv = 1
      lww = 1
      mxv = maxv
      mxw = maxw
      do inst = 1, nparn

            ! Restore nesting partition matrix
         rewind junit1
         read(junit1) zpart
            ! Condition matrix is restricted to current nesting part
         lexs = zpart == inst

         call pacalf( expf, nexpf, gtall, balang, balant,            &
                      flag, mxv, mxw, lexs, rcoef, nrco,             &
                      ityp, rtyp, kdeg, nrsavd,                      &
                      nregc, intcp, prop, ndf0, zinter, iord0,       &
                      ngpar, ntpar, npar0,                           &
                      zgpar, ztpar, zpart,                           &
                      vv(:,lvv:lvv+mxv-1), ww(:,lww:lww+mxw-1), lerr )
            ! Restore condition matrix for next comparison
         if ( lerr .or. inst == nparn ) read(junit1) lexs

         if ( lerr ) goto 10

            ! Note that nested comparisons are currently restricted
            ! to product-form ones, so that this section takes account
            ! of this when computing df's
         if ( ityp > 3 ) then
!!          call chkpg( 2, .false., newp )
            write(lgu,*) ermrk1
            write(lgu,888)
            write(lgu,*) ermrk2
!!          lines = lines + 2
            goto 10
         end if
         if ( inst > 1 ) then
            if ( ityp /= cmptyp .or. iord0 /= iord .or. min(npar0,2) /= npart ) then
!!             call chkpg( 5, .false., newp )
               write(lgu,*) ermrk1
               write(lgu,999)
               write(lgu,*) ermrk2
!!             lines = lines + 5
               goto 10
            end if
               ! Check for redundancy among contrasts
            newp = .true.
            if ( ngrp > 1 ) newp = valrdy( vv(:,lvv:), vv, ngrp, ncv, nvdf, tzero )
            if ( newp ) ncv = 0
            newp = .true.
            if ( ntrt > 1 ) newp = valrdy( ww(:,lww:), ww, ntrt, ncw, nwdf, tzero )
            if ( newp ) ncw = 0
         else
            cmptyp = ityp
            npart  = min( npar0, 2 )
            iord   = iord0
         end if

         nvdf = nvdf + ncv
         nwdf = nwdf + ncw
         ndf1 = ndf1 + ndf0
         lvv = lvv + ncv
         lww = lww + ncw
         mxv = maxv - lvv + 1
         mxw = maxw - lww + 1

      end do
      ncv = nvdf
      ncw = nwdf

         ! Check for an overall product type comparison
      if ( ndf1 /= ncv*ncw ) then
!!       call chkpg( 3, .false., newp )
         write(lgu,*) ermrk1
         write(lgu,1111)
         write(lgu,*) ermrk2
!!       lines = lines + 3
         goto 10
      end if
         ! Get a common norm for the contrasts (harmonic mean of norms)
      if ( ncv > 1 ) call pachmc( vv, ngrp, ncv, ag )
      if ( ncw > 1 ) call pachmc( ww, ntrt, ncw, at )

         ! Reset npart and nrsavd to 0 thus indicating that part array
         ! and regression-variables (if any) are no longer available,
         ! and restore original lpmean
      npart  = 0
      nrsavd = 0
      lpmean = zpmean

   end if

   lupvw = cmptyp /= 4
      ! Allocate and fill in v and w
   if ( lupvw ) then
      nszu = ncw
   else
      nszu = ngrp
   end if
   if ( allocated(v) ) deallocate(v)
   if ( allocated(w) ) deallocate(w)
   allocate ( v(ngrp,ncv), w(ntrt,nszu), stat=status )
   if ( status /= 0 ) then   ! fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacDir)."
      write(lgu,*) ermrk2
      call gmp999            ! release memory space and files
      call pac999( 99 )
      stop
   end if
   v = vv(:,1:ncv)
   w = ww(:,1:nszu)
      ! Release vv and ww
   if ( allocated(vv) ) deallocate(vv)
   if ( allocated(ww) ) deallocate(ww)

   ! Normal return
   ! -------------
   if ( lupvw .and. .not. lspecf ) then
      ik = 1
   else
      ik = 0
   end if
   nladd = 0
   if ( intcp .or. rtyp /= 0 ) then
      nladd = (lrecp-1-1) / 14
      nladd = min( nladd, 9 )
      fmtr(5:5) = achar( krnk0+nladd )
      nladd = 3 + (nregc-1) / nladd + 1
      call chkpg( nladd+3+ik, .false., newp )
      rmsg = ' '
      k    = 1
      if ( intcp ) then
         rmsg(k:) = mint
         k = k + len( mint )
      else if ( prop ) then
         rmsg(k:) = mpro
         k = k + len( mpro )
      end if
      if ( abs(rtyp) == 2 ) then
         rmsg(k:) = msgr(2)
         k = k + len( msgr(2) )
      end if
      if ( rtyp < 0 ) rmsg(k:) = msgr(1)
      write(imp,555) kdeg, rmsg
      write(imp,fmtr) rcoef(1:nregc)
      lines = lines + nladd
   else
      call chkpg( 3+ik, .false., newp )
   end if
   write(imp,666) ctyp(cmptyp), ntyp(nestyp), ndf1
   if ( lupvw .and. .not. lspecf ) write(imp,777)
   lines = lines + 3 + ik
   if ( ndf1 <= 0 ) goto 10
      ! Print warning on validity conditions if relevant
   if ( nestyp /= 0 .and. nestyp /= 2 .and. ncw > 1 .and. cmptyp > 1 ) then
      call chkpg( 6, .false., newp )
      write(imp,2222)
      lines = lines + 6
   end if

   return

   ! Error return
   ! ------------
10 continue
   lerr = .true.
   lpmean = zpmean
   if ( allocated(v) ) deallocate(v)
   if ( allocated(w) ) deallocate(w)
   if ( allocated(vv) ) deallocate(vv)
   if ( allocated(ww) ) deallocate(ww)

   INCLUDE 'allocf.'
   INCLUDE 'pacforf.'

END SUBROUTINE pacfor
