SUBROUTINE pacmv2( core, epsl, nc, nr, m, iuw, lmult, cstdf, cstb, cstc,  &
                   prtm, amat, namv, nnamv, baylev, kova, kodev, kodcir,  &
                   boik, namb, ierr )
!***********************************************************************
!     - pacmv2 -
!     computes and prints MultiVariate statistics and tests
!     assuming equal covariances matrices
!***********************************************************************
!
!     CORE  - INPUT . The core matrix to be diagonalized,     - REAL*8
!                     of order NC, stored in triangular form
!             OUTPUT. Destroyed
!     EPSL  - INPUT . A small real constant used to compute   - REAL*8
!                     relative zero tolerance for eigenvalues
!                     as EPSL * trace(CORE) / NC
!     NC    - INPUT . Order of the CORE matrix, i.e. number   - INTEGER
!                     of what's treated as dependent variables
!                     (thus number of columns in AMAT if used)
!     NR    - INPUT . Number of rows in AMAT                  - INTEGER
!     M     - INPUT . Univariate numerator degrees of freedom - INTEGER
!                     If M = 1 then all multivariate tests are
!                     equivalent
!     IUW   - INPUT . Unit-number for saving eigen values and - INTEGER
!                     vectors of CORE when LRDUCE is .true.
!                     (must not be rewinded)
!                     If negative, no save
!     LMULT - INPUT . If .false. and data reduction step is   - LOGICAL
!                     requested then only diagonalization is  - LOGICAL
!                     performed (and results saved on IUW),
!                     no further computations nor outputs
!     CSTDF - INPUT . The within df associated to the         - REAL*8
!                     comparison on relevant groups
!     CSTB  - INPUT . A coefficient involved in computation   - REAL*8
!                     of scale index of Bayes distribution
!                     It is the square of component of norm
!                     of G-contrast vectors (or 1.0 when the
!                     procedure applies to slopes of dep. var.
!                     for covariates) (May be multiplied by a
!                     constant if covariates)
!                     Note: if covariates adjustment applies
!                     and NR is 1, then it was divided by a
!                     corrective scalar
!                     Meaningful only if the comparison is
!                     product form
!     CSTC  - INPUT . A coefficient used to properly rescale  - REAL*8
!                     eigenvalues of the core matrix to be
!                     diagonalized (must be one if already
!                     scaled)
!                     Meaningful only if the comparison is
!                     product form
!     PRTM  - INPUT . .true. if matrix AMAT, used to compute  - LOGICAL
!                     CORE, is to be printed
!                     N.B. to be effective printing also
!                     requires NR = 1
!     AMAT  - INPUT . The matrix from which the core matrix   - REAL*8
!                     was computed, of length NR*NC
!                     Used only if PRTM is .true.
!     NAMV  - INPUT . Names of the NC dependent variables     - CHAR
!                     Used only if M=NR=1 and PRTM is .true.
!     NNAMV - INPUT . Dimension of NAMV                       - INTEGER
!     BAYLEV- INPUT . Bayes-probability level                 - REAL*8
!                     Currently used only if M = 1
!     KOVA  - INPUT . Status code regarding covariates:       - INTEGER
!                       0 : procedure applies to dep. var.
!                           with no covariates
!                       1 : procedure applies to dep. var.
!                           adjusted by covariates
!                      -1 : procedure applies to slopes of
!                           dep. var. for covariates
!     KODEV - INPUT . Code number for hypothesis of equality  - INTEGER
!                     of the covariances matrices
!                     0 : not assumed
!                     1 : assumed
!     KODCIR- INPUT . Code number for circularity hypothesis  - INTEGER
!                     about the covariances matrices
!                     0 : not assumed
!                     1 : assumed
!     BOIK  - INPUT . Boik's epsilon (1991) for circularity   - REAL*8
!                       = 0.0 if irrelevant,
!                       < 0.0 if unavailable
!     NAMB  - INPUT . A label used to represent the set of    - CHAR
!                     dependent variables when a Bayes save
!                     file is used (must be of same length as
!                     the variable names NOMV)
!     IERR  - OUTPUT. 0 if no error                           - INTEGER
!                     >0 : error in diagonalization :
!                     eigenvalues in WRK3(IERR+1)..WRK3(NC)
!                     (and corresponding eigenvectors)
!                     may be presumed inaccurate
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     pacint : ranka
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG  PACFID  PRTTAM  TQL2X
!     External functions called:
!        BETACDF
!     Fortran  functions called:
!        ABS  MIN  SQRT
!
!-----------------------------------------------------------------------

   USE gmpint
   USE gmplgl
   USE pacdtr
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Arguments
   !  ---------

   REAL(KIND=8), INTENT(IN) :: epsl, cstdf, cstb, cstc, baylev, boik
   INTEGER, INTENT(IN) :: nc, nr, m, iuw, nnamv, kova, kodev, kodcir
   LOGICAL, INTENT(IN) :: lmult, prtm
   CHARACTER(LEN=*), INTENT(IN) :: namb
   REAL(KIND=8), INTENT(IN) :: amat(nr,nc)
   CHARACTER(LEN=*), INTENT(IN) :: namv(nnamv)
   REAL(KIND=8), INTENT(INOUT) :: core(nc*(nc+1)/2)
   INTEGER, INTENT(OUT) :: ierr

   !  Local declarations
   !  ------------------

   REAL(KIND=8), EXTERNAL :: betacdf

   REAL(KIND=8), PARAMETER :: zero=0.0_8, half=0.5_8, one=1.0_8, two=2.0_8, four=4.0_8, five=5.0_8
   REAL(KIND=8) :: b, cb, df1m, df2m, df2t, efcal, ftest, ftrce, prob, prot
   REAL(KIND=8) :: qkst, rlkhd, rnc, rootl, scale, test, tolz, trace, tracx
   INTEGER :: i, ierb, ij, j, k, kapp, kb, minmnc, neig, nkova, nvar, nvv
   LOGICAL :: lrduw, newp
   CHARACTER(LEN=4) :: nob
   REAL(KIND=8) :: wrk2(nc,nc), wrk3(nc), wrk4(nc)
   CHARACTER(LEN=2) :: mapp(0:3)=(/ '  ', ' a', ' A', ' A' /)
   CHARACTER(LEN=8) :: blan(1)=(/ '        ' /)
      ! blan is an array used to ensure consistency with array
      ! argument in PRTTAM

   INCLUDE 'pacmv2p.'

   !--------------------------------------------------------------------

      ! efcal is numerical indicator of standardized effect magnitude
      !    Multivariate test statistics:
      ! trace is Trace criterion  (Lawley & Hotelling T0**2)
      ! rlkhd is Likelihood ratio (Wilks)
      ! rootl is Largest root     (Roy)

   df1m = m*nc   ! Numerator degrees of freedom
   rnc = nc

   ! Compute trace of core
   ! ---------------------
   trace = zero
   ij = 0
   do i = 1, nc
      ij = ij + i
      trace = trace + core(ij)
   end do

   tolz = epsl * trace / nc

   lrduw = lrduce .and. iuw >= 0

   if ( m == 1 ) then
         ! Rank of core is one, although its order is nc
         ! If data reduction step requested, diagonalize and save info
      if ( lrduw ) then
         ranka = 1
         call tql2x( nc, core, wrk3, wrk4, wrk2, lrduw, ierr )
         write(iuw) nc, 1, ierr, trace, wrk3(1), tolz
         write(iuw) wrk3(1)
         write(iuw) (wrk2(i,1),i=1,nc)
      end if
      if ( .not. lmult ) return

      ! Compute equivalent F-test only, as all tests are equivalent
      ! -----------------------------------------------------------
         ! ftest is F statistic
         ! kapp  is a flag for approximation of distribution
      efcal = trace / df1m
      df2m  = cstdf - df1m + one
      kapp  = 0
      if ( df2m > zero ) then
            ! Significance level of F-statistic
         scale = cstdf*cstb / df2m
         ftest = efcal / scale
         efcal = sqrt( efcal )
         scale = sqrt( scale )
         test  = df1m * ftest
         prob  = betacdf( test/(test+df2m), half*df1m, half*df2m, ierb )
         if ( ierb == 0 ) then
            prob = one - prob
         else
            prob = -one
         end if
      else
         efcal = sqrt( efcal )
         scale = zero
      end if

   else

      ! Compute multivariate test-statistics
      ! ------------------------------------
         ! Diagonalize core (minmnc is the maximum rank)
      minmnc = min( m, nc )
      call tql2x( nc, core, wrk3, wrk4, wrk2, lrduw, ierr )
      if ( ierr == 0 ) then
            ! Multivariate test-statistics
         tracx = wrk3(1)
         rlkhd = tracx*cstc + one
         rootl = tracx*cstc / rlkhd
         neig  = 1
         do i = 2, nc
            if ( wrk3(i) <= tolz ) exit
            rlkhd = rlkhd * (wrk3(i)*cstc + one)
            neig  = neig + 1
            tracx = tracx + wrk3(i)
         end do
            ! Save information for data reduction step
         if ( lrduw ) then
            ranka = neig
            write(iuw) nc, minmnc, ierr, trace, tracx, tolz
            if ( neig < nc ) then
               write(iuw) (wrk3(i),i=1,neig)
               write(iuw) ((wrk2(i,j),i=1,nc),j=1,neig)
            else
               write(iuw) wrk3
               write(iuw) wrk2
            end if
         end if
            ! Check numerical accuracy of trace
         if ( neig > minmnc .or. abs(trace-tracx) > tolz ) then
            call chkpg( 4, .false., newp )
            write(imp,111) minmnc, neig, trace, tracx
            lines = lines + 4
            return
         end if
      else
         if ( lrduw ) write(iuw) nc, minmnc, ierr, trace, zero, tolz
         if ( lmult ) then
            call chkpg( 2, .false., newp )
            write(imp,888) ierr
            lines = lines + 2
            return
         end if
      end if
         ! That's all if multivariate test-statistics not requested
      if ( .not. lmult ) return

      efcal = sqrt( trace / df1m )
      trace = trace * cstdf * cstc
      rlkhd = one / rlkhd

         ! Compute exact or approximate F-statistic for rlkhd
      if ( minmnc <= 2 ) then
            ! Exact distribution
         kapp = 0
         df2m = minmnc * (cstdf - (nc-1))
         if ( minmnc == 2 ) then
            ftest = sqrt(rlkhd)
         else
            ftest = rlkhd
         end if
      else
            ! Approximate distribution
         kapp  = 2
         df2m  = cstdf  +  half * (m-(nc+1))
         b     = sqrt( ((m*m)*(nc*nc)-four) / ((m*m)+(nc*nc)-five) )
         df2m  = df2m * b -  half * (m*nc-2)
         ftest = rlkhd**(one/b)
      end if
      ftest = df2m/df1m * (one-ftest)/ftest
         ! Significance level of F-statistic
      if ( df2m > zero ) then
         test = df1m * ftest
         prob = betacdf( test/(test+df2m), half*df1m, half*df2m, ierb )
         if ( ierb == 0 ) then
            prob = one - prob
         else
            prob = -one
         end if
      else
         prob = -one
      end if

         ! Compute approximate F-statistic for trace
      if ( cstdf > nc+3 ) then
         df2t  = cstdf - (nc+1)
         b     = cstdf * df1m / df2t
         test  = (cstdf+one) / (cstdf-nc) * b * ((cstdf+cstdf)*(df2t+m)) / (df2t*(df2t-two))
         df2t  = (df1m*test*two+b*b*(df1m-two)) / (df1m*test-b*b*two) * two
         test  = df2t / ((df2t-two)*b)
         ftrce = trace * test
      else
         df2t  = zero
         ftrce = zero
      end if
         ! Significance level of F-statistic
      if ( df2t > zero ) then
         test = df1m * ftrce
         prot = betacdf( test/(test+df2t), half*df1m, half*df2t, ierb )
         if ( ierb == 0 ) then
            prot = one - prot
         else
            prot = -one
         end if
      else
         prot = -one
      end if

   end if

   ! Compute bayes limits if requested and print all results
   ! -------------------------------------------------------

   cb = sqrt( cstb )

   if ( prtm ) then
      k = 6
   else
      k = 3
   end if
   if ( m > 1 ) k = k + 2
!!    remplacer if ( m > 1 )... ci-dessus par:               ! Boik
!! if ( m > 1 ) then                                         ! Boik
!!    k = k + 2                                              ! Boik
!!    if ( boik > zero .and. boik < one ) k = k + 7          ! Boik
!! end if                                                    ! Boik

   nob = '    '
   kb  = 0
   if ( baylev > zero ) then
      kb = 3
      if ( m > 1 ) kb = 1
      if ( lbayes .and. (m > 1 .or. df2m > zero) ) then
            ! Save Bayes-fiducial information
         if ( kova /= 0 ) then
            nkova = ncova
         else
            nkova = 0
         end if
         if ( kodcir == 0 ) then
            nvar = nc / ncw
            qkst = cstdf
         else
            nvar = nc
            qkst = cstdf / ncw
         end if
         nvv = ncv
         if ( kova == -1 ) nvv = ncv*ncw
         nbayes = nbayes + 1
         write(nob,9999) nbayes
         write(ibayes,8888) nob, nvar, nkova, kova, kodev, kodcir,  &
                            namb, efcal, one, cb, cb, qkst, nvv, ncw
      end if
   end if

   if ( m == 1 ) then

         ! Print standardized effect if required
         ! Note that amat is transmitted to prttam as its own
         ! transpose, and thus it works only when AMAT has 1 row
      if ( prtm .and. nr == 1 ) then
         call chkpg( 5, .false., newp )
         write(imp,777)
         lines = lines + 2
         call prttam( imp, .true., amat, nc, nc, nr, namv(1:nc), blan )
      end if
         ! Note that kb lines are to be reserved for Bayes outputs
      call chkpg( k+kb, .false., newp )
      write(imp,222) efcal
      lines = lines + 3
      if ( df2m > zero ) then
            ! Print F-test
         write(imp,333) nob, msgscl, msgscu
         write(imp,444) scale, ftest, df1m, df2m, prob, mapp(kapp)
         lines = lines + 3
            ! Compute and print Bayes-fiducial inferences
         if ( kb > 0 ) call pacfid( baylev, efcal, cb, zero, rnc, cstdf,  &
                                    .false., .false., 0, m, wrk4(1) )
      else
         write(imp,666)
         lines = lines + 1
      end if

   else
      call chkpg( k+kb, .false., newp )
      write(imp,222) efcal
      lines = lines + 3
      write(imp,333) nob, msgsta, msgstu
      write(imp,555)                                         &
         pwilks, rlkhd, ftest, df1m, df2m, prob, mapp(kapp), &
         ptrace, trace, ftrce, df1m, df2t, prot, mapp(2),    &
         pevmax, rootl
      lines = lines + 5
         ! Compute and print Bayes-fiducial inferences
         ! Note that only (true-obs) deviation for effect is computed
      if ( kb > 0 ) call pacfid( baylev, efcal, cb, zero, rnc, cstdf,  &
                                 .false., .false., 0, m, wrk4(1) )
!!       ! Boik's correction for circularity
!!    if ( boik > zero .and. boik < one ) then                        ! Boik
!!       write(imp,999)                                               ! Boik
!!       lines = lines + 3                                            ! Boik
!!       df1m =                                                       ! Boik
!!       df2m =                                                       ! Boik
!!       prob = -one                                                  ! Boik
!!       df2t =                                                       ! Boik
!!       prot = -one                                                  ! Boik
!!       write(imp,555)                                         &     ! Boik
!!          pwilks, rlkhd, ftest, df1m, df2m, prob, mapp(kapp), &     ! Boik
!!          ptrace, trace, ftrce, df1m, df2t, prot, mapp(2)           ! Boik
!!       lines = lines + 2                                            ! Boik
!!       write(imp,333) '    ', msgsta, msgstu                        ! Boik
!!          ! And also with Huynh-Feldt adjustment                      ! Boik
!!       df1m =                                                       ! Boik
!!       df2m =                                                       ! Boik
!!       prob = -one                                                  ! Boik
!!       df2t =                                                       ! Boik
!!       prot = -one                                                  ! Boik
!!       write(imp,1111)                                             &! Boik
!!        '(HF)', pwilks, rlkhd, ftest, df1m, df2m, prob, mapp(kapp),&! Boik
!!        '(HF)', ptrace, trace, ftrce, df1m, df2t, prot, mapp(2)     ! Boik
!!       lines = lines + 2                                            ! Boik
!!    end if                                                          ! Boik

   end if

   INCLUDE 'pacmv2f1.'
   INCLUDE 'pacmv2f.'

END SUBROUTINE pacmv2
