SUBROUTINE pacppm( balang, balant, npart, rtyp, nvect, nvct1, part,    &
                   gapp, xbar, at, gwght, twght, labg, labt, nomv )
!***********************************************************************
!     - pacppm -
!     Prints the Partition matrix and the associated Means
!***********************************************************************
!
!     BALANG- INPUT . .true. if equal group counts            - LOGICAL
!                     (design is balanced for groups)
!     BALANT- INPUT . .true. if equal treatment weights       - LOGICAL
!                     (design is balanced for treatments)
!     NPART - INPUT . Number of parts in the partition matrix - INTEGER
!     RTYP  - INPUT . Type of regression:                     - INTEGER
!                        0 = no regression requested
!                        1 = degree type
!                        2 = cumulative type
!                     (a negative type codes for the residual
!                     comparison)
!     NVECT - INPUT . Number of regression-variables vectors  - INTEGER
!                     saved on scratch unit IU4
!     NVCT1 - INPUT . Maximum ( 1, NVECT )                    - INTEGER
!     PART  - INPUT . The partition matrix resulting from     - INTEGER
!                     formula analysis,
!                     2-dimensioned (NTRT, NGRP)
!     GAPP  - INPUT .    see definition of MEM2 sub-arrays    - INTEGER
!     XBAR  - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     AT    - INPUT .    see definition of MEM2 sub-arrays    - REAL*8
!     GWGHT - INPUT . Vector of group weights, depending on   - REAL*8
!                     the weighting type of the formula,
!                     of length NGRP
!     TWGHT - INPUT . Vector of treatment weights, depending  - REAL*8
!                     on the weighting type of the formula,
!                     of length NTRT
!     LABG  - INPUT .    see definition of MEMB sub-arrays    - CHAR
!     LABT  - INPUT .    see definition of MEMB sub-arrays    - CHAR
!     NOMV  - INPUT .    see definition of MEMB sub-arrays    - CHAR
!     NOMV2 - WORK ARRAY of length 1 + k*NVTOT                - CHAR
!                     where k is 1 if NVECT=0, and 2 otherwise
!     NOMP  - WORK ARRAY of length NPART                      - CHAR
!     PWEI  - WORK ARRAY of length NPART                      - REAL*8
!     DADJ  - WORK ARRAY 2-dimensioned (NVCT1, NVTOT)         - REAL*8
!                     used to store effects
!                     unused if NVECT=0
!     REGR  - WORK ARRAY 2-dimensioned (NPART, NVCT1)         - REAL*8
!                     used to store regression-variables
!                     unused if NVECT=0
!     PBAR  - WORK ARRAY 2-dimensioned (NPART, 0:*)           - REAL*8
!                     The actual number of columns used is
!                     1+NVTOT if NVECT=0 (no regression) and
!                     1+2*NVTOT otherwise
!     A     - WORK ARRAY of length m*(m+1)/2 where m <= NPART - REAL*8
!                     (the actual value of m is KLAST+1,
!                     KLAST being read in this routine)
!                     unused if NVECT=0
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CHKPG   GMP999  PAC999  PRTTAM
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ALLOCATE  ALLOCATED  DEALLOCATE  DOT_PRODUCT  INT  LEN_TRIM
!        LEN_TRIM  LOG10  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE pacint
   USE paclgl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: npart, rtyp, nvect, nvct1
   LOGICAL, INTENT(IN) :: balang, balant
   REAL(KIND=8), INTENT(IN) :: xbar(nlxb,ngrp), at(ntrt)
   REAL(KIND=8), INTENT(IN) :: gwght(ngrp), twght(ntrt)
   INTEGER, INTENT(IN) :: part(ntrt,ngrp), gapp(ngrp)
   CHARACTER(LEN=*), INTENT(IN) :: labg(ngrp), labt(ntrt), nomv(nvtot)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: sumx, tot
   INTEGER :: i, incr, iout, iresd, j, k, kdeg, klast, ko, k2
   INTEGER :: l, line2, ll, maxil, mintl
   INTEGER :: nbar, nbeg, ndigit, nend, nladd, nlr, nmor, pik, status
   LOGICAL :: lchk, more, newp, sava
   CHARACTER(LEN=25) :: fmtp='(1x,a,1x,"|",   i .0,"|")'
   CHARACTER(LEN=12) :: fmtt='(   x,200a )'
   REAL(KIND=8), ALLOCATABLE :: a(:)
   REAL(KIND=8) :: pwei(npart), dadj(nvct1,nvtot), regr(npart,nvct1)
   REAL(KIND=8) :: pbar(npart,0:nvtot+min(nvect,1)*nvtot)
   CHARACTER(LEN=nocw) :: nomv2(0:nvtot+min(nvect,1)*nvtot)
   CHARACTER(LEN=nocw) :: nomp(npart)

      ! nocw (in gmpint) is the length of G and T labels and of variables names
      ! (if not 8, formats must be modified accordingly)

   INCLUDE 'pacppmp.'

   !--------------------------------------------------------------------
      ! pbar usage: the part weights are stored in column 0 and the
      ! part means in other columns; if regression, the observed
      ! means are stored in columns 1, 3, 5,... and the corresponding
      ! adjusted means in columns 2, 4, 6,...

   if ( nvect > 0 ) then
      incr = 2
   else
      incr = 1
   end if
   nbar = incr * nvtot

   ! Create the matrix of part-means and the part names
   ! --------------------------------------------------
      ! Initialize
   pwei = zero
   pbar = zero
   do i = 1, npart
      write(nomp(i),555) nampar, i
   end do
      ! Calculate weighted sum of cell means. And also part weights
      ! using arithmetic derivation (thus part weights are appropriate
      ! only if equal weights stands for both G's and T's)
   sumx = xbar(nlxb,gapp(1))
   do k = 1, ngrp
      ko = gapp(k)
      do i = 1, ntrt
         pik = part(i,k)
         if ( pik > 0 ) then
            pwei(pik)   = pwei(pik) + gwght(k)*twght(i)
            pbar(pik,0) = pbar(pik,0) + xbar(nlxb,ko)*at(i)
            l = i
            do j = 1, nbar, incr
               pbar(pik,j) = pbar(pik,j) + gwght(k)*twght(i)*xbar(l,ko)
               l = l + ntrt
            end do
         end if
      end do
   end do
      ! Final step: part means
   do j = 1, nbar, incr
      pbar(1:npart,j) = pbar(1:npart,j) / pwei(1:npart)
   end do

      ! If needed, calculate part weights using the correct derivation
      ! (according to the metric used in pacalf)

   if ( .not. ( (balang .and. balant)         .or.  &
                (gpond == 2 .and. tpond == 2) .or.  &
                (balang .and. tpond == 2)     .or.  &
                (balant .and. gpond == 2) ) ) then

      pwei(1:npart) = zero
      pbar(1:npart,0) = zero

      if ( gpond == 2 ) then
            ! Group size weights for G
         if ( .not. balant .and. tpond /= 2 ) then
            do k = 1, ngrp
               ko = gapp(k)
               do i = 1, ntrt
                  pik = part(i,k)
                  if ( pik > 0 ) then
                     pwei(pik)   = pwei(pik) + xbar(nlxb,ko)
                     pbar(pik,0) = pbar(pik,0) + xbar(nlxb,ko)/at(i)
                  end if
               end do
            end do
         end if

      else
            ! Equal weights for G
         if ( .not. balang .and. tpond == 2 ) then
            do k = 1, ngrp
               ko = gapp(k)
               do i = 1, ntrt
                  pik = part(i,k)
                  if ( pik > 0 ) then
                     pwei(pik)   = pwei(pik) + at(i)
                     pbar(pik,0) = pbar(pik,0) + at(i)/xbar(nlxb,ko)
                  end if
               end do
            end do
         else if ( .not. (balang .and. balant) .and. tpond /= 2 ) then
            do k = 1, ngrp
               ko = gapp(k)
               do i = 1, ntrt
                  pik = part(i,k)
                  if ( pik > 0 ) then
                     pwei(pik)   = pwei(pik) + one
                     pbar(pik,0) = pbar(pik,0) + one/(at(i)*xbar(nlxb,ko))
                  end if
               end do
            end do
         end if
      end if
      pbar(1:npart,0) = pwei(1:npart) * pwei(1:npart) / pbar(1:npart,0)

   end if

   ! Create variable names, and calculate adjusted means if regression
   ! -----------------------------------------------------------------
      ! Note that weight is considered as variable no. 0
   nomv2(0) = namsiz
   sava = .false.
   if ( nvect > 0 ) then

      if ( rtyp >= 0 ) then
         iresd = 0
      else
         iresd = 1
      end if
         ! Recover elements from which the contrasts (measures) were created:
         ! Read in pwei the reciprocal of the metric terms (they should
         ! be identical to pbar(.,0)), and in regr the variables
      rewind iu4
      read(iu4) pwei
      read(iu4) klast, kdeg, sava
      if ( sava ) then
            ! Note that degree 0 must be taken in account (thus going
            ! up to klast+1 and kdeg+1 as here the arrays start with
            ! index 1 instead of 0)
         k = klast + 1
         allocate ( a(1:k*(k+1)/2), stat=status ) ! Allocate work array
         if ( status /= 0 ) then   ! Fatal error
            write(lgu,*) ermrk1
            write(lgu,101) " (PacPPM)."
            write(lgu,*) ermrk2
            call gmp999            ! Release memory space and files
            call pac999( 99 )
            stop
         end if
         j = 0
         do i = 1, klast+1
            read(iu4) (a(k),k=j+1,j+i)
            j = j + i
         end do
      end if
      read(iu4) regr

      do j = 1, nvect
            ! Calculate effects in dadj
         sumx = zero
         do i = 1, npart
            sumx = sumx + pwei(i) * regr(i,j) * regr(i,j)
         end do
         k2 = -1
         do k = 1, nvtot
            k2  = k2 + 2
            tot = zero
            do i = 1, npart
               tot = tot  +  pwei(i) * regr(i,j) * pbar(i,k2)
            end do
               ! N.B. sumx may be zero in case of proportionality
            if ( sumx > zero ) then
               dadj(j,k) = tot / sumx
            else
               dadj(j,k) = zero
            end if
         end do
      end do
         ! Calculate adjusted means
      k2 = 0
      do k = 1, nvtot
         k2 = k2 + 2
         do i = 1, npart
            pbar(i,k2) = dot_product( dadj(1:nvect,k), regr(i,1:nvect) )
         end do
         nomv2(k2-1) = nomv(k)
         nomv2(k2)   = namadj(iresd)
      end do

   else
      nomv2(1:nvtot) = nomv(1:nvtot)

   end if

   ! Print partition matrix
   ! ----------------------
   if ( lptabl ) then

         ! Look for minimal length of T-labels
      mintl = 1
      do i = 1, ntrt
         j = len_trim( labt(i) )
         if ( j > mintl ) then
            mintl = j
            if ( mintl >= nocw ) exit
         end if
      end do

      call chkpg( 3+mintl+3, .false., newp )
      write(imp,111)
      lines = lines + 3

         ! ndigit is the minimum necessary number of digits to print
         !        part numbers
         ! maxil  is the maximum number of items printed per line

      tot = npart
      ndigit = int( log10(tot) ) + 1
      write(fmtt(2:4),222) 1+nocw+2
      write(fmtt(11:11),'(i1)') ndigit
      fmtp(18:18) = fmtt(11:11)
      maxil  = (lrecp-(3+nocw)-1) / ndigit

      lchk = .true.
      more = .false.
      nmor = mintl + 1
      nend = 0
      do
         nbeg = nend + 1
         nend = min( nend+maxil, ntrt )
         write(fmtp(14:16),222) nend-nbeg+1

         if ( lchk ) call chkpg( nmor+2, .false., newp )
         if ( more ) write(imp,*) ' '
         do j = 1, mintl
            write(imp,fmtt) (labt(i)(j:j),i=nbeg,nend)
         end do
         write(imp,333) ('-',i=1,(nend-nbeg+1)*ndigit+2)
         lines = lines + nmor
            ! nlr   = no. of lines still to be printed
            ! nladd = no. of lines in current page
         nlr = ngrp
         ll = 0
         do
            if ( lchk ) then
               nladd = min( nlr, max( linep-lines-1, 1 ) )
            else
               nladd = nlr
            end if
            do l = 1, nladd
               ll = ll + 1
               write(imp,fmtp) labg(ll), (part(i,ll),i=nbeg,nend)
            end do
            write(imp,333) ('-',i=1,(nend-nbeg+1)*ndigit+2)
            lines = lines + nladd + 1
            nlr = nlr - nladd
            if ( nlr <= 0 ) exit
            if ( lchk ) call chkpg( 0, .true., newp )
         end do

         more = .true.
         nmor = mintl + 2
         if ( nend >= ntrt ) exit
      end do

      if ( lchk ) line2 = lines
      lchk = .false.
      lines = line2

   end if

   ! Print part means
   ! ----------------
   nladd = 4
   if ( nvect > 0 ) nladd = 5
   call chkpg( 3+nladd, .false., newp )
   write(imp,444)
   lines = lines + 3
   iout  = imp
   call prttam( iout, .true., pbar, npart, npart, 1+nbar, nomp, nomv2 )

   ! If regression calculate and print coefficients of the polynom
   ! -------------------------------------------------------------
   if ( sava ) then
         ! Calculate the coefficients (store in pbar)
      k2 = kdeg + 1
      do i = 1, k2
         if ( rtyp > 1 ) then
            pbar(i,1:nvtot) = zero
            do j = i, k2
               l = j*(j-1)/2 + i
               do k = 1, nvtot
                  pbar(i,k) = pbar(i,k)  +  dadj(j,k) * a(l)
               end do
            end do
         else
            l = k2*kdeg/2 + i
!!!!        if ( i > 1 .or. kdeg == 0 ) then
            if ( i > 1 ) then
               do k = 1, nvtot
                  pbar(i,k) = dadj(nvect,k) * a(l)
               end do
            else
               do k = 1, nvtot
                  pbar(i,k) = dadj(1,k)  +  dadj(nvect,k) * a(l)
               end do
            end if
         end if
      end do
      if ( allocated(a) ) deallocate(a)   ! release work array
         ! Create coefficient names
      do i = 0, kdeg
         write(nomp(i+1),555) namdeg, i
      end do
         ! Print
      call chkpg( 6, .false., newp )
      write(imp,666)
      lines = lines + 3
      call prttam( iout, .true., pbar(1:,1:), npart, k2, nvtot, nomp, nomv )
   end if

   INCLUDE 'allocf.'
   INCLUDE 'pacppmf.'

END SUBROUTINE pacppm
