SUBROUTINE pacreg( kdeg, rtyp, intcp, prop, ns1, nc1, ns2, np1,       &
                   rcoef, par1, par2, wgh1, cco1, cco2, ncco1, nrsavd )
!***********************************************************************
!     - pacreg -
!     computes the marginal contrast-coefficients associated to a
!     polynomial REGression (or its residual) on a product form
!     between-parts comparison of type 1 or 2.
!     The set (G or T) on which regression applies is called the
!     1st set, and the other one is called the 2nd set.
!***********************************************************************
!
!     KDEG  - INPUT . Degree of the polynomial regression     - INTEGER
!     RTYP  - INPUT . Type   of the polynomial regression     - INTEGER
!                     1 = degree KDEG only
!                     2 = cumulative degrees up to KDEG
!                     A negative type codes for the residual
!                     comparison
!     INTCP - INPUT . .true. if intercept is requested        - LOGICAL
!     PROP  - INPUT . .true. if proportionality is requested  - LOGICAL
!     NS1   - INPUT . Number of elements (rows) in the 1st set- INTEGER
!     NC1   - INPUT . Maximum number of columns in the 1st set- INTEGER
!     NS2   - INPUT . Number of elements in the 2nd set       - INTEGER
!     NP1   - INPUT . Number of differents parts in PAR1      - INTEGER
!     RCOEF - INPUT . Vector of regression coefficients       - REAL*8
!                     of length NP1
!     PAR1  - INPUT . Marginal-partition vector for 1st set,  - INTEGER
!                     of length NS1
!     PAR2  - INPUT . Marginal-partition vector for 2nd set,  - INTEGER
!                     of length NS2
!     WGH1  - INPUT . Vector of marginal part-weights,        - REAL*8
!                     of length NP1.
!                     It is reciprocal of the metric used to
!                     get orthogonal contrasts.
!     CCO1  - OUTPUT. Matrix of contrast coefficients for the - REAL*8
!                     1st set, 2-dimensioned (NS1,NC1)
!     CCO2  - OUTPUT. Vector of contrast coefficients for the - REAL*8
!                     2nd set, 1-dimensioned (NS2)
!     NCCO1 - OUTPUT. Number of vectors stored in CCO1,       - INTEGER
!                     i.e. df of the comparison
!     NRSAVD- OUTPUT. Number of regression-variables vectors  - INTEGER
!                     saved on scratch unit.
!                     Relevant only if LPMEAN is .true. and
!                     INTCP is .false. (set to 0 otherwise)
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        GMP999  PAC999  POLREG
!     External functions called:
!        NONE
!     Fortran  functions called:
!        ALLOCATE  ALLOCATED  DEALLOCATE
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE memst2
   USE paclgl

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: kdeg, rtyp, ns1, nc1, ns2, np1
   LOGICAL, INTENT(IN) :: intcp, prop
   INTEGER, INTENT(IN) :: par1(ns1), par2(ns2)
   INTEGER, INTENT(OUT) :: ncco1, nrsavd
   REAL(KIND=8), INTENT(IN) :: rcoef(np1), wgh1(np1)
   REAL(KIND=8), INTENT(OUT) :: cco1(ns1,nc1), cco2(ns2)

   !  Local declarations
   !  ------------------

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8, onem=-one
   INTEGER :: i, j, k, klast, k1st, status, zdeg, zrtyp
   LOGICAL :: zprop
   REAL(KIND=8) :: co1(0:2)=(/ zero, one, onem /)
   REAL(KIND=8), ALLOCATABLE :: wrk(:,:)

   !--------------------------------------------------------------------
      ! Note.
      ! When intercept is requested,
      ! - if NP1 = 1, it is equivalent to "mean";
      ! - otherwise it is not directly calculated but equivalenced to
      !   residual to proportionality, and then the only one relevant
      !   contrast (degree 0) is stored.

   ncco1  = 0
   nrsavd = 0

   ! Get unique contrast for 2nd set
   ! -------------------------------
   do i = 1, ns2
      cco2(i) = co1( par2(i) )
   end do

   zprop = intcp .or. prop
   if ( intcp .and. np1 > 1 ) then
      zrtyp = -1
      zdeg  = 1
   else
      zrtyp = rtyp
      zdeg  = kdeg
   end if

   ! Get polynomial regression coefficients up to appropriate degree
   ! ---------------------------------------------------------------
   if ( zrtyp > 0 ) then
      k1st  = 1
      if ( zrtyp == 1 .or. zdeg == 0 ) k1st = zdeg
      klast = zdeg
   else
      k1st  = zdeg + 1
      klast = np1 - 1
         ! Check it is worth continuing.
         ! Note that if np1 is 2 and residual to proportionality
         ! is requested, then degree 0 is to be calculated, and this
         ! implies calculation of degree 1 too, although k1st > klast
      if ( .not. zprop .and. (k1st > klast) ) then
         if ( zrtyp == -2 .or. np1 <= 2 ) return
         klast = klast - 1
      end if
   end if

      ! Allocate work array
   allocate ( wrk(1:np1,0:klast), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacReg)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if

      ! Get contrast-coefficients up to degree klast in wrk
   call polreg( klast, zdeg, zrtyp, zprop, intcp, np1, rcoef, wgh1,   &
                lpmean .and. .not.intcp, iu4, nrsavd, wrk )

   ! Fill in contrast vectors for 1st set
   ! ------------------------------------
   if ( zrtyp == -1 ) then
         ! Residual to zdeg: fill in with degrees 1 to zdeg-1
         ! or with degree 0 if residual to proportionality
      if ( zprop ) then
         j = 0
      else
         j = 1
      end if
      do k = j, zdeg-1
         ncco1 = ncco1 + 1
         do i = 1, ns1
            if ( par1(i) == 0 ) then
               cco1(i,ncco1) = zero
            else
               cco1(i,ncco1) = wrk(par1(i),k)
            end if
         end do
      end do
   end if
   if ( .not. intcp .or. (intcp .and. np1 == 1) ) then
         ! Fill in with degrees k1st to klast
      do k = k1st, klast
         ncco1 = ncco1 + 1
         do i = 1, ns1
            if ( par1(i) == 0 ) then
               cco1(i,ncco1) = zero
            else
               cco1(i,ncco1) = wrk(par1(i),k)
            end if
         end do
      end do
   end if

   if ( allocated(wrk) ) deallocate(wrk)

   INCLUDE 'allocf.'

END SUBROUTINE pacreg
