SUBROUTINE pacstx( ngmix, zmin, zmax, nxv, lfgrp, lonce, ltmis, lmisx, lerr )

!***********************************************************************
!     - pacstx -
!     computes elementary STatistics (mean, std-deviation, min & max)
!     on basic protocol "X" for each group, treatment and variable,
!     and stores data on scratch unit with convenient group-number
!     and covariate measures
!***********************************************************************
!
!     NGMIX - INPUT . Maximum manageable number of groups     - INTEGER
!     ZMIN  - INPUT . Minima values of grouping-variables     - REAL*8
!                     Relevant only if LFGRP is .true.
!     ZMAX  - INPUT . Maxima values of grouping-variables     - REAL*8
!                     Relevant only if LFGRP is .true.
!     NXV   - INPUT . Number of dep.var. + (unrepeated) cova. - INTEGER
!     LFGRP - INPUT . .true. if NGFA0 > 0                     - LOGICAL
!     LONCE - INPUT . .true. if cova. are given only once     - LOGICAL
!                     when T > 1 (i.e. constant over T)
!     LTMIS - OUTPUT. .true. when at least one treatment is   - LOGICAL
!                     neither complete nor entirely missing
!                     for at least one group
!     LMISX - OUTPUT. .true. when there are missing data      - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     memst2 : xbar  ctab  gcod  gmax
!     pacint : ngrp
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        GMP999  PAC999
!     External functions called:
!        NUMGX
!     Fortran  functions called:
!        ALLOCATE  ALLOCATED  DEALLOCATE  MAX  NINT  RESHAPE  SQRT
!
!----------------------------------------------------------------------
!
!     NOTE: Organization of xbar (2-dimensional array)
!           The rows refer to statistics and the columns to groups in
!           order of appearance in data.
!           The number of rows is  nlxb = 4*ntdc + 1
!           The number of columns is the actual number of groups (ngrp).
!           X-means are stored in the first ntdc rows (treatments vary
!           faster); X-std-deviations in the ntdc following ones;
!           minima in the ntdc following ones; maxima in the ntdc
!           following ones;
!           group-counts are stored in the last row (i.e. row nlxb).
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE memst2
   USE pacint
   USE paclgl
   USE spcval

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   INTEGER, INTENT(IN) :: ngmix, nxv
   LOGICAL, INTENT(IN) :: lfgrp, lonce
   REAL(KIND=8), INTENT(IN) :: zmin(ngfac), zmax(ngfac)
   LOGICAL, INTENT(OUT) :: ltmis, lmisx, lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: numgx

   REAL(KIND=8), PARAMETER :: zero=0.0_8, one=1.0_8
   REAL(KIND=8) :: dev, frq, rdl
   INTEGER :: i, ij, ijk, ijkv, j, jj, jk, k, kase, ko, kv, mg
   INTEGER :: ncount, nlxbk, nlxbk1, ntdc2, num, nxvtc, nxvtd, status
   CHARACTER(LEN=incdw) :: slab
   REAL(KIND=8) :: wxbar(nlxb*ngmix)
   INTEGER :: wctab(ntdc,ngmix), wgcod(ngfac,ngmix), wgmax(ngmix)
   REAL(KIND=8) :: xinp(nvaru), xgv(nuvar), xg(ngfac), xvr(ntdc)
      ! xinp is work vector for all variables of case on input
      ! xgv  is work vector for variables used (group.var.+dep.var.+cova.)
      ! xg   is work vector for grouping variables
      ! xvr  is work vector for dependent var. + covariates
      !                     with cova. being repeated for T if they
      !                     were given only once (see lonce)

   !--------------------------------------------------------------------

   lerr  = .false.
   ntdc2 = ntdc + ntdc
   nxvtd = nxv - ncova
   nxvtc = nxvtd + ntrt*ncova

      ! Initialize
   wxbar = zero
   kv = ntdc2 - nlxb
   do k = 1, ngmix
      kv = kv + nlxb
      do ij = 1, ntdc
         wctab(ij,k) = 0
         ijk = ij + kv
         wxbar(ijk)      = xout
         wxbar(ijk+ntdc) = -xout
      end do
   end do

   rewind iu1
   rewind iu2
   wgmax(1) = 1
   wgcod(1,1)= 1
   ngrp = 0

   do kase = 1, nindvu
      read(iu1) num, slab, frq, xinp

         ! Reorder variables according to usev
      do i = 1, nuvar
         xgv(i) = xinp( usev(i) )
      end do
      xg = xgv(1:ngfac)
      xvr(1:nuvar-ngfa0) = xgv(ngfa0+1:nuvar)

         ! ko is group-number according to appearance-order in data
      ko = 1

      if ( lfgrp ) then

            ! Compute group-number according to maximum group-table
         mg = numgx( ngfac, xg, zmin, zmax )
            ! Find group-number according to appearance-order in data
         do k = 1, ngrp
            if ( mg == wgmax(k) ) goto 10
            ko = ko + 1
            if ( ko > ngmix ) then
               write(lgu,*) ermrk1
               write(lgu,111) ngmix
               write(lgu,*) ermrk2
               lerr = .true.
               return
            end if
         end do
            ! Update gmax and gcod
         ngrp = ko
         wgmax(ko) = mg
         do i = 1, ngfac
            wgcod(i,ko) = nint( xg(i) )
         end do
      10 continue

      end if

         ! Replicate covariates if given only once (constant over T)
      if ( lonce ) then
         jk = nxvtc
         do j = ncova, 1, -1
            jj = nxvtd + j
            jk = jk - ntrt
            do k = jk+1, jk+ntrt
               xvr(k) = xvr(jj)
            end do
         end do
      end if

         ! Save data with appearance-order group-number & group.var.
      if ( lfgrp ) then
         write(iu2) num, slab, frq, ko, xg,  xvr
      else
         write(iu2) num, slab, frq, ko, one, xvr
      end if

         ! Compute means and standard-deviations (using BMDP's
         ! provisional means algorithm for sake of accuracy of
         ! sum-of-squares)
         ! and update matrix of counts of measures (ctab)
      nlxbk  = nlxb * ko
      nlxbk1 = nlxbk - nlxb
      ncount = nint( max(frq,zero) )
      if ( ncount > 0 ) then
         frq = ncount  ! Ensure consistency of count and frequency
         wxbar(nlxbk) = wxbar(nlxbk) + frq
         do ij = 1, ntdc
            if ( xvr(ij) < xout ) then
               wctab(ij,ko) = wctab(ij,ko) + ncount
               ijk = ij + nlxbk1
               dev = frq * ( xvr(ij) - wxbar(ijk) )
               wxbar(ijk)      = wxbar(ijk) + dev/wxbar(nlxbk)
               wxbar(ijk+ntdc) = wxbar(ijk+ntdc) + dev*(xvr(ij)-wxbar(ijk))
               ijkv = ijk + ntdc2
               if ( xvr(ij) < wxbar(ijkv) ) wxbar(ijkv) = xvr(ij)
               if ( xvr(ij) > wxbar(ijkv+ntdc) ) wxbar(ijkv+ntdc) = xvr(ij)
            end if
         end do
      end if

   end do

   if ( ngrp <= 0 ) ngrp = 1

      ! Final computations for standard-deviations
      ! Note that treatments that have not full count are considered
      ! missing for the concerned group, and their mean and std.dev
      ! are reset to zero and -xout
   ltmis  = .false.
   lmisx  = .false.
   nlxbk1 = -nlxb
   do ko = 1, ngrp
      nlxbk1 = nlxbk1 + nlxb
      nlxbk  = nlxbk1 + nlxb
      ncount = nint( wxbar(nlxbk) )
      if ( ncount > 1 ) then
         rdl = one / ( wxbar(nlxbk)-one )
         do ij = 1, ntdc
            ijk = ij + nlxbk1
            if ( wctab(ij,ko) == ncount ) then
               wxbar(ijk+ntdc) = wxbar(ijk+ntdc)*rdl
               if ( wxbar(ijk+ntdc) > zero ) then  ! Protect against negative zero-like values
                  wxbar(ijk+ntdc) = sqrt( wxbar(ijk+ntdc) )
               else
                  wxbar(ijk+ntdc) = zero
               end if
            else
               wxbar(ijk)      = zero
               wxbar(ijk+ntdc) = -xout
               ltmis = ltmis .or. wctab(ij,ko) > 0
               lmisx = .true.
            end if
         end do
      else
            ! Group count is only one
         wxbar(nlxbk1+ntdc+1:nlxbk1+ntdc2) = -xout
      end if
   end do

      ! Create arrays
   if ( allocated(xbar) ) deallocate(xbar)
   if ( allocated(ctab) ) deallocate(ctab)
   if ( allocated(gcod) ) deallocate(gcod)
   if ( allocated(gmax) ) deallocate(gmax)
   allocate ( xbar(nlxb,ngrp), ctab(ntdc,ngrp), gcod(ngfac,ngrp), gmax(ngrp), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacStX)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if
   xbar = reshape( wxbar(1:nlxb*ngrp), (/nlxb,ngrp/) )
   ctab = wctab(1:ntdc,1:ngrp)
   gcod = wgcod(1:ngfac,1:ngrp)
   gmax = wgmax(1:ngrp)

   INCLUDE 'allocf.'
   INCLUDE 'pacstxf.'

END SUBROUTINE pacstx
