SUBROUTINE pacx( lonce, lablg, lablt, gsym, tsym, templ, allgt, lerr )

!***********************************************************************
!     - pacx -
!     directs general computations on basic data set "X"
!***********************************************************************
!
!     LONCE - INPUT . Whether covariates were measured once,  - LOGICAL
!                     so are to be replicated over treatments
!     LABLG - INPUT . Whether G-labels were entered by user   - LOGICAL
!     LABLT - INPUT . Whether T-labels were entered by user   - LOGICAL
!     GSYM  - INPUT . Array of symbols for G-factors          - CHAR
!     TSYM  - INPUT . Array of symbols for T-factors          - CHAR
!     TEMPL - INPUT . Array of G-labels of length NGUSER      - CHAR
!                     if LABLG is .true. (unused otherwise)
!     ALLGT - OUTPUT. Whether the G&T cross-table is complete - LOGICAL
!     LERR  - OUTPUT. .true. if an error is detected          - LOGICAL
!
!     --------------------------------------------------------
!     Module elements modified:
!     gmpint : lines
!     memst2 : hyp0  xlim  ag  gnum  gorg  gapp  gstr  gmod  tmod
!              lgtx
!     memstb : labg  labt
!     pacint : ngmax  nlxb
!     paclgl : lmima  limit
!
!-----------------------------------------------------------------------
!
!     Subroutines called:
!        CONSEC  GMP999  PACLAB  PACMIS  PACPRX  PACSGT  PACSTX  PAC999
!     External functions called:
!        NUMGX
!     Fortran  functions called:
!        ALLOCATE  ALLOCATED  DEALLOCATE  MAX  MIN
!
!-----------------------------------------------------------------------

   USE gmpchr
   USE gmpint
   USE gmplgl
   USE memst1
   USE memst2
   USE memsta
   USE memstb
   USE pacint
   USE paclgl
   USE spcval

   IMPLICIT NONE

   !  Interface block
   !  ---------------

   INTERFACE
      SUBROUTINE pac999( u1, u2, u3 )
         IMPLICIT NONE
         INTEGER, INTENT(IN), OPTIONAL :: u1, u2, u3
      END SUBROUTINE pac999
   END INTERFACE

   !  Arguments
   !  ---------

   LOGICAL, INTENT(IN) :: lonce, lablg, lablt
   CHARACTER(LEN=*), INTENT(IN) :: gsym(ngfac), tsym(ntfac), templ(nguser)
   LOGICAL, INTENT(OUT) :: allgt, lerr

   !  Local declarations
   !  ------------------

   INTEGER, EXTERNAL :: numgx

   REAL(KIND=8), PARAMETER :: zero=0.0_8
   INTEGER :: i, iout, j, k, ko, ngmix, status
   LOGICAL :: lfgrp, lmisx, ltmis
   REAL(KIND=8) :: wrka(ngfac), wrkb(ngfac)

   !--------------------------------------------------------------------

   lerr = .false.
   iout = imp

   lfgrp = ngfa0 > 0

   if ( allocated(hyp0) ) deallocate(hyp0)
   if ( allocated(xlim) ) deallocate(xlim)
   if ( allocated(gmod) ) deallocate(gmod)
   if ( allocated(tmod) ) deallocate(tmod)
   allocate ( hyp0(nvtot), xlim(nvtot), gmod(ngfac), tmod(ntfac), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacX)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if

      ! Compute maximum number of groups according to extrema values
      ! of grouping-variables
   if ( lfgrp ) then
         ! Store in wrka and wrkb extrema of grouping-variables,
         ! according to usev order
      do i = 1, ngfac
         wrka(i) = ymin(usev(i))
         wrkb(i) = ymax(usev(i))
      end do
      ngmax = numgx( ngfac, wrkb, wrka, wrkb )
      ngmix = min( ngmax, nguser )
   else
      ngmax = 1
      ngmix = 1
   end if
   lmima = .true.
   nlxb = 4*ntdc + 1

   ! Compute means, std-deviations, min, max on X
   ! --------------------------------------------
      ! (ngrp is known on return from pacstx)
   call pacstx( ngmix, wrka, wrkb, nuvar-ngfa0, lfgrp, lonce, ltmis, lmisx, lerr )
      ! If error analysis is aborted
   if ( lerr ) return

   ! Allocate arrays
   ! ---------------
   if ( allocated(ag) ) deallocate(ag)
   if ( allocated(gnum) ) deallocate(gnum)
   if ( allocated(gorg) ) deallocate(gorg)
   if ( allocated(gapp) ) deallocate(gapp)
   if ( allocated(gsup) ) deallocate(gsup)
   if ( allocated(gstr) ) deallocate(gstr)
   if ( allocated(lgtx) ) deallocate(lgtx)
   if ( allocated(lres) ) deallocate(lres)
   if ( allocated(labg) ) deallocate(labg)
   allocate ( ag(ngrp), gnum(ngrp), gorg(ngrp), gapp(ngrp), gsup(ngrp),  &
              gstr(ngrp,ngfac), lgtx(ntrt,ngrp), lres(ntrt,ngrp),        &
              labg(ngrp), stat=status )
   if ( status /= 0 ) then   ! Fatal error
      write(lgu,*) ermrk1
      write(lgu,101) " (PacX)."
      write(lgu,*) ermrk2
      call gmp999            ! Release memory space and files
      call pac999( 99 )
      stop
   end if

   ! Initialize hyp0 and xlim (and thus limit)
   ! -----------------------------------------
   limit = .false.
   hyp0 = zero
   xlim = zero

   ! Create gnum, gorg, gapp
   ! -----------------------
   if ( lfgrp ) then
         ! Create gnum
      gnum = gmax
      call consec( gnum, ngrp, 0, maxint, k, j )
         ! Create gorg and gapp
      do k = 1, ngrp
         j = gnum(k)
         gorg(j) = gmax(k)
         gapp(j) = k
      end do
   else
      gnum(1) = 1
      gorg(1) = 1
      gapp(1) = 1
   end if

   ! Reorder group-counts into ag
   ! ----------------------------
   do k = 1, ngrp
      ag(gnum(k)) = xbar(nlxb,k)
   end do

   ! Check missing data pattern
   ! --------------------------
      ! If missing data, correct unauthorized ones if any, and
      ! check for consistency of their pattern for all variables
   if ( lmisx ) call pacmis( ltmis, ctab, gapp, ag, lerr )
      ! If error analysis is aborted
   if ( lerr ) return

   ! Create lgtx, gstr, gmod, tmod and set up allgt
   ! ----------------------------------------------
      ! (Remember that the G&T structure is supposed to be the same
      ! for each variable, so that only the subset of ctab which
      ! corresponds to the 1st variable is used to create lgtx)
      ! Create lgtx and allgt and copy transpose(gcod), in appropriate
      ! order, in gstr
   allgt = .true.
   do ko = 1, ngrp
      k = gnum(ko)
      do i = 1, ntrt
         lgtx(i,k) = ctab(i,ko) > 0
         allgt = allgt .and. lgtx(i,k)
      end do
      do i = 1, ngfac
         gstr(k,i) = gcod(i,ko)
      end do
   end do
      ! Now, to obtain gstr replace grouping-factor codes by consecutive
      ! numbers, starting from 1, for each one of the grouping-factors
      ! At the same time, get number of categories of grouping-factors
   do i = 1, ngfac
      call consec( gstr(:,i), ngrp, 0, maxint, k, j )
      gmod(i) = j
   end do
      ! Do the same for tstr and categories of treatment-factors
      ! (Note that the value 0 is not transformed and not included in
      ! the number of actual categories)
   do i = 1, ntfac
      call consec( tstr(:,i), ntrt, 0, maxint, k, j )
      tmod(i) = j
   end do

   ! Create G and T labels
   ! ---------------------
   if ( lablg ) then
      labg = templ(1:ngrp)
   else
      if ( ngfa0 > 0 ) then
         call paclab( labg, gsym, gmod, gstr, ngfac, ngrp, 'g' )
      else
         labg(1) = '        '
      end if
   end if
   if ( .not. lablt ) then
      if ( ntrt > 1 ) then
         call paclab( labt, tsym, tmod, tstr, ntfac, ntrt, 'o' )
      else
         labt(1) = '        '
      end if
   end if

   ! Print G&T-structure
   ! -------------------
   call pacsgt( iout, .true., gsym, tsym, lonce, allgt )

   ! Print X-statistics if requested
   ! -------------------------------
   if ( lstax ) call pacprx( iout, .true., lfgrp )

   INCLUDE 'allocf.'
   INCLUDE 'pacxf.'

END SUBROUTINE pacx
